@@ -6,6 +6,7 @@ fed into our Bison parser, that is shared with the free-form parser.
 
 Note: The prescanner removes CR, so we only handle LF here.
 */
+#include <unordered_map>
 #include <utility>
 
 #include <lfortran/parser/parser_exception.h>
@@ -22,7 +23,7 @@ int position = 0;
 
 namespace LCompilers::LFortran {
 
-std::map<std::string, yytokentype> identifiers_map = {
+const std::unordered_map<std::string, yytokentype> identifiers_map = {
     {"EOF", END_OF_FILE},
     {"\n", TK_NEWLINE},
     {"name", TK_NAME},
@@ -266,15 +267,21 @@ std::map<std::string, yytokentype> identifiers_map = {
     {"uminus", UMINUS}
 };
 
-std::vector<std::string> declarators{
+// star-forms must appear before non-stars
+const std::vector<std::string> declarators{
+            "integer*",
             "integer",
+	    "real*",
             "real",
+	    "complex*",
             "complex",
             "doubleprecision",
             "doublecomplex",
             "external",
             "dimension",
+	    "character*",
             "character",
+	    "logical*",
             "logical",
             "bytes",
             "data",
@@ -431,7 +438,7 @@ struct FixedFormRecursiveDescent {
     // Push the token_type, YYSTYPE and Location of the token_str at `cur`.
     // (Does not modify `cur`.)
     void push_token_no_advance_token(unsigned char *cur, const std::string &token_str,
-            yytokentype token_type) {
+            yytokentype const token_type) {
         YYSTYPE yy;
         yy.string.from_str(m_a, token_str);
         stypes.push_back(yy);
@@ -444,7 +451,9 @@ struct FixedFormRecursiveDescent {
 
     // token_type automatically determined
     void push_token_no_advance(unsigned char *cur, const std::string &token_str) {
-        push_token_no_advance_token(cur, token_str, identifiers_map[token_str]);
+	auto it = identifiers_map.find(token_str);
+	LCOMPILERS_ASSERT(it != identifiers_map.end());
+        push_token_no_advance_token(cur, token_str, it->second);
     }
 
     void push_integer_no_advance(unsigned char *cur, int32_t n) {
@@ -719,7 +728,7 @@ struct FixedFormRecursiveDescent {
         ptrdiff_t len;
         while (t.cur < end) {
             YYSTYPE y2;
-            auto token = t.lex(m_a, y2, loc, diag);
+            auto token = t.lex(m_a, y2, loc, diag, false);
             len = t.cur - t.tok;
             tokens.push_back(token);
             if (token == yytokentype::TK_INTEGER) {
@@ -841,7 +850,20 @@ struct FixedFormRecursiveDescent {
     bool lex_declarator(unsigned char *&cur) {
         for(const auto& declarator : declarators) {
             if(next_is(cur, declarator)) {
-                push_token_advance(cur, declarator);
+		if (declarator.back() == '*') {
+		    /* In "star" form declarations, we need to be careful of only parsing an integer (no kind)
+		       after the star, so that we correctly parse "character*2d3v" as "character*2 d3v"
+		       instead of "character*2d3 v", which is illegal. */
+		    push_token_advance(cur, declarator.substr(0, declarator.size() - 1));
+		    push_token_advance(cur, std::string("*"));
+		    unsigned char *int_start = cur;
+		    if (try_integer(cur)) {
+			int32_t val = std::atoi((char*)int_start);
+			push_integer_no_advance(int_start, val);
+		    }
+		} else {
+		    push_token_advance(cur, declarator);
+		}
                 return true;
             }
         }
@@ -855,7 +877,7 @@ struct FixedFormRecursiveDescent {
                     cur += io_str.size();
                     unsigned char *start;
                     Location loc;
-                    lex_format(cur, loc, start);
+                    lex_format(cur, loc, start, diag, false, this->string_start);
                     locations.push_back(loc);
                     YYSTYPE yylval;
                     yylval.string.p = (char*) start;
@@ -991,7 +1013,7 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    bool lex_body_statement(unsigned char *&cur) {
+    bool lex_body_statement(unsigned char *&cur, bool continue_compilation = false) {
         int64_t l = eat_label(cur);
         // handle derived type tokenization
         // this needs to be done before 'lex_declaration'
@@ -1058,6 +1080,24 @@ struct FixedFormRecursiveDescent {
             return true;
         }
 
+        if (next_is(cur, "backspace")) {
+            push_token_advance(cur, "backspace");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "rewind")) {
+            push_token_advance(cur, "rewind");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "endfile")) {
+            push_token_advance(cur, "endfile");
+            tokenize_line(cur);
+            return true;
+        }
+
         if (next_is(cur, "interface")) {
             lex_interface(cur);
             return true;
@@ -1164,6 +1204,13 @@ struct FixedFormRecursiveDescent {
             // Undo the label, as it will be handled later
             undo_label(cur);
         }
+        if (next_is(cur, "endprogram") || next_is(cur, "end") || next_is(cur, "contains") || next_is(cur, "subroutine") || next_is(cur, "function")) {
+            return false;
+        }
+        if (continue_compilation) {
+            tokenize_line(cur);
+            return true;
+        }
 
         return false;
     }
@@ -1493,6 +1540,12 @@ struct FixedFormRecursiveDescent {
             push_token_no_advance(cur, "continue");
             push_token_no_advance(cur, "\n");
         }
+        if (next_is(cur, "contains")) {
+            push_token_advance(cur, "contains");
+            push_token_no_advance(cur, "\n");
+            next_line(cur);
+            while(lex_procedure(cur));
+        }
         if (next_is(cur, "endsubroutine")) {
             push_token_advance(cur, "endsubroutine");
             tokenize_line(cur);
@@ -1519,18 +1572,36 @@ struct FixedFormRecursiveDescent {
                    end
             ```
     */
-    void lex_program(unsigned char *&cur, bool explicit_program) {
+    void lex_program(unsigned char *&cur, bool explicit_program, bool continue_compilation, diag::Diagnostics &diagnostics) {
         if (explicit_program) {
             push_token_advance(cur, "program");
             tokenize_line(cur);
         }
-        while(lex_body_statement(cur));
+        while(lex_body_statement(cur, continue_compilation));
         eat_label(cur);
         if (next_is(cur, "contains")) {
             push_token_advance(cur, "contains");
             push_token_no_advance(cur, "\n");
             next_line(cur); // Does not generate any code?
             while(lex_procedure(cur));
+        } else if (next_is(cur, "subroutine") || next_is(cur, "function")) {
+            Location loc;
+            loc.first = cur - string_start;
+            loc.last = cur - string_start;
+            if (continue_compilation) {
+                diagnostics.add(LFortran::parser_local::TokenizerError("Expecting contains keyword before procedure definition", loc).d);
+                if (next_is(cur, "subroutine")) {
+                    while (!next_is(cur, "endprogram")) {
+                        next_line(cur);
+                    }
+                } else if (next_is(cur, "function")) {
+                    while (!next_is(cur, "endprogram")) {
+                        next_line(cur);
+                    }
+                }
+            } else {
+                throw parser_local::TokenizerError("Expecting contains keyword before procedure definition", loc);
+            }
         }
         if (next_is(cur, "endprogram")) {
             push_token_advance(cur, "endprogram");
@@ -1724,7 +1795,7 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    void lex_global_scope_item(unsigned char *&cur) {
+    void lex_global_scope_item(unsigned char *&cur, bool continue_compilation, diag::Diagnostics &diagnostics) {
         // we can define a global assignment
         unsigned char *nline = cur; next_line(nline);
         // eat_label(cur);
@@ -1733,7 +1804,7 @@ struct FixedFormRecursiveDescent {
             tokenize_line(cur);
         }
         if (is_program(cur)) {
-            lex_program(cur, true);
+            lex_program(cur, true, continue_compilation, diagnostics);
         } else if (next_is(cur, "interface")) {
             lex_interface(cur);
         } else if (is_module(cur)) {
@@ -1747,18 +1818,18 @@ struct FixedFormRecursiveDescent {
             push_token_no_advance(cur, "program");
             push_token_no_advance_token(cur, "implicit_program_lfortran", TK_NAME);
             push_token_no_advance(cur, "\n");
-            lex_program(cur, false);
+            lex_program(cur, false, continue_compilation, diagnostics);
         } else {
             error(cur, "ICE: Cannot recognize global scope entity");
         }
     }
 
-    void lex_global_scope(unsigned char *&cur) {
+    void lex_global_scope(unsigned char *&cur, bool continue_compilation, diag::Diagnostics &diagnostics) {
         auto next = cur;
         while (*cur != '\0') {
             // eat_label(cur);
             next_line(next);
-            lex_global_scope_item(cur);
+            lex_global_scope_item(cur, continue_compilation, diagnostics);
             next = cur;
         }
         push_token_no_advance(cur, "EOF");
@@ -1766,7 +1837,7 @@ struct FixedFormRecursiveDescent {
 
 };
 
-bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocator &al) {
+bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocator &al, bool continue_compilation) {
     // We use a recursive descent parser.  We are starting at the global scope
     try {
         FixedFormRecursiveDescent f(diagnostics, al);
@@ -1776,7 +1847,7 @@ bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocato
         f.t.string_start = string_start;
         f.t.cur_line = string_start;
         f.t.line_num = 1;
-        f.lex_global_scope(cur);
+        f.lex_global_scope(cur, continue_compilation, diagnostics);
         tokens = std::move(f.tokens);
         stypes = std::move(f.stypes);
         locations = std::move(f.locations);
@@ -1791,7 +1862,7 @@ bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocato
 }
 
 int FixedFormTokenizer::lex(Allocator &/*al*/, YYSTYPE &yylval,
-        Location &loc, diag::Diagnostics &/*diagnostics*/)
+        Location &loc, diag::Diagnostics &/*diagnostics*/, bool /*continue_compilation*/)
 {
     if (!tokens.empty()) {
         auto tok = tokens[token_pos];
