@@ -196,6 +196,14 @@ class ASRBuilder {
         return EXPR(ASR::make_IntegerConstant_t(al, loc, x, t));
     }
 
+    inline ASR::expr_t* logical_true() {
+        return EXPR(ASR::make_LogicalConstant_t(al, loc, true, logical));
+    }
+
+    inline ASR::expr_t* logical_false() {
+        return EXPR(ASR::make_LogicalConstant_t(al, loc, false, logical));
+    }
+
     inline ASR::expr_t* i32(int64_t x) {
         return EXPR(ASR::make_IntegerConstant_t(al, loc, x, int32));
     }
@@ -287,7 +295,7 @@ class ASRBuilder {
     }
 
     inline ASR::expr_t* ArraySize(ASR::expr_t* x, ASR::expr_t* dim, ASR::ttype_t* t) {
-        return EXPR(make_ArraySize_t_util(al, loc, x, dim, t, nullptr));
+        return EXPR(make_ArraySize_t_util(al, loc, x, dim, t, nullptr, false));
     }
 
     inline ASR::expr_t* Ichar(std::string s, ASR::ttype_t* type, ASR::ttype_t* t) {
@@ -769,6 +777,9 @@ class ASRBuilder {
             case ASR::ttypeType::Logical: {
                 return EXPR(ASR::make_LogicalCompare_t(al, loc, left, ASR::cmpopType::Eq, right, logical, nullptr));
             }
+            case ASR::ttypeType::Complex: {
+                return EXPR(ASR::make_ComplexCompare_t(al, loc, left, ASR::cmpopType::Eq, right, logical, nullptr));
+            }
             default: {
                 throw LCompilersException("Expression type, " +
                     ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
@@ -856,6 +867,8 @@ class ASRBuilder {
     }
 
     ASR::expr_t *ArrayItem_01(ASR::expr_t *arr, std::vector<ASR::expr_t*> idx) {
+        // LCOMPILERS_ASSERT(
+        //     ASRUtils::extract_n_dims_from_ttype(ASRUtils::expr_type(arr)) == idx.size());
         Vec<ASR::expr_t*> idx_vars; idx_vars.reserve(al, 1);
         for (auto &x: idx) idx_vars.push_back(al, x);
         return PassUtils::create_array_ref(arr, idx_vars, al);
@@ -902,7 +915,7 @@ class ASRBuilder {
     ASR::stmt_t *Assignment(ASR::expr_t *lhs, ASR::expr_t *rhs) {
         LCOMPILERS_ASSERT_MSG(check_equal_type(expr_type(lhs), expr_type(rhs)),
             type_to_str_python(expr_type(lhs)) + ", " + type_to_str_python(expr_type(rhs)));
-        return STMT(ASR::make_Assignment_t(al, loc, lhs, rhs, nullptr));
+        return STMT(ASRUtils::make_Assignment_t_util(al, loc, lhs, rhs, nullptr, false));
     }
 
     ASR::stmt_t* CPtrToPointer(ASR::expr_t* cptr, ASR::expr_t* ptr, ASR::expr_t* shape = nullptr, ASR::expr_t* lower_bounds = nullptr) {
@@ -974,6 +987,25 @@ class ASRBuilder {
         return STMT(ASR::make_DoLoop_t(al, loc, nullptr, head, body.p, body.n, nullptr, 0));
     }
 
+    /*
+    if loop_body contains A(i, j, k) then set idx_vars=(k, j, i)
+    in order to iterate on the array left to right, the inner-most
+    loop on the fastest index on the left, as is common in Fortran
+    ```
+        idx_vars=(k, j, i)
+        body A(i, j, k)
+
+        produces
+
+        do k = 1, n
+            do j = 1, n
+                do i = 1, n
+                    A(i, j, k)
+                end do
+            end do
+        end do
+    ```
+    */
     template <typename LOOP_BODY>
     ASR::stmt_t* create_do_loop(
         const Location& loc, int rank, ASR::expr_t* array,
@@ -982,7 +1014,7 @@ class ASRBuilder {
         PassUtils::create_idx_vars(idx_vars, rank, loc, al, scope, "_i");
 
         ASR::stmt_t* doloop = nullptr;
-        for( int i = (int) idx_vars.size() - 1; i >= 0; i-- ) {
+        for ( int i = 0; i < (int) idx_vars.size(); i++ ) {
             ASR::do_loop_head_t head;
             head.m_v = idx_vars[i];
             head.m_start = PassUtils::get_bound(array, i + 1, "lbound", al);
@@ -1009,7 +1041,7 @@ class ASRBuilder {
         Vec<ASR::stmt_t*>& doloop_body, LOOP_BODY loop_body) {
 
         ASR::stmt_t* doloop = nullptr;
-        for( int i = (int) loop_vars.size() - 1; i >= 0; i-- ) {
+        for ( int i = 0; i < (int) loop_vars.size(); i++ ) {
             ASR::do_loop_head_t head;
             head.m_v = loop_vars[i];
             head.m_start = PassUtils::get_bound(array, loop_dims[i], "lbound", al);
