@@ -1,4 +1,3 @@
-#include <iostream>
 #include <memory>
 
 #include <libasr/asr.h>
@@ -259,7 +258,7 @@ public:
                 std::string encoded_type_name = "i" + std::to_string(t->m_kind * 8);
                 handle_array(t2, type_name, true)
             } else {
-                diag.codegen_error_label("Type number '"
+                diag.codegen_error_label("Type '"
                     + ASRUtils::type_to_str_python(v.m_type)
                     + "' not supported", {v.base.base.loc}, "");
                 throw Abort();
@@ -306,7 +305,7 @@ public:
                 sub = format_type_c("", list_type_c, v.m_name,
                                     false, false);
             } else {
-                diag.codegen_error_label("Type number '"
+                diag.codegen_error_label("Type '"
                     + ASRUtils::type_to_str_python(v.m_type)
                     + "' not supported", {v.base.base.loc}, "");
                 throw Abort();
