@@ -53,6 +53,7 @@
 #endif
 #if LLVM_VERSION_MAJOR >= 17
     // TODO: removed from LLVM 17
+    #include <llvm/Passes/PassBuilder.h>
 #else
 #    include <llvm/Transforms/IPO/PassManagerBuilder.h>
 #endif
@@ -101,6 +102,16 @@ std::string LLVMModule::str()
     return LLVMEvaluator::module_to_string(*m_m);
 }
 
+llvm::Function *LLVMModule::get_function(const std::string &fn_name) {
+    llvm::Module *m = m_m.get();
+    return m->getFunction(fn_name);
+}
+
+llvm::GlobalVariable *LLVMModule::get_global(const std::string &global_name) {
+    llvm::Module *m = m_m.get();
+    return m->getNamedGlobal(global_name);
+}
+
 std::string LLVMModule::get_return_type(const std::string &fn_name)
 {
     llvm::Module *m = m_m.get();
@@ -147,6 +158,7 @@ MLIRModule::MLIRModule(std::unique_ptr<mlir::ModuleOp> m,
         std::unique_ptr<mlir::MLIRContext> ctx) {
     mlir_m = std::move(m);
     mlir_ctx = std::move(ctx);
+    llvm_ctx = std::make_unique<llvm::LLVMContext>();
 }
 
 MLIRModule::~MLIRModule() {
@@ -154,21 +166,27 @@ MLIRModule::~MLIRModule() {
     llvm_ctx.reset();
 };
 
-std::string MLIRModule::str() {
+std::string MLIRModule::mlir_str() {
     std::string mlir_str;
     llvm::raw_string_ostream raw_os(mlir_str);
     mlir_m->print(raw_os);
     return mlir_str;
 }
 
-void MLIRModule::mlir_to_llvm() {
-    llvm_ctx = std::make_unique<llvm::LLVMContext>();
+std::string MLIRModule::llvm_str() {
+    std::string mlir_str;
+    llvm::raw_string_ostream raw_os(mlir_str);
+    llvm_m->print(raw_os, nullptr);
+    return mlir_str;
+}
+
+void MLIRModule::mlir_to_llvm(llvm::LLVMContext &ctx) {
     std::unique_ptr<llvm::Module> llvmModule = mlir::translateModuleToLLVMIR(
-        *mlir_m, *llvm_ctx);
+        *mlir_m, ctx);
     if (llvmModule) {
         llvm_m = std::move(llvmModule);
     } else {
-        throw std::runtime_error("Failed to generate LLVM IR");
+        throw LCompilersException("Failed to generate LLVM IR");
     }
 }
 #endif
@@ -392,15 +410,26 @@ void LLVMEvaluator::opt(llvm::Module &m) {
     m.setTargetTriple(target_triple);
     m.setDataLayout(TM->createDataLayout());
 
+#if LLVM_VERSION_MAJOR >= 17
+    llvm::LoopAnalysisManager LAM;
+    llvm::FunctionAnalysisManager FAM;
+    llvm::CGSCCAnalysisManager CGAM;
+    llvm::ModuleAnalysisManager MAM;
+    llvm::PassBuilder PB = llvm::PassBuilder(TM);
+    PB.registerModuleAnalyses(MAM);
+    PB.registerCGSCCAnalyses(CGAM);
+    PB.registerFunctionAnalyses(FAM);
+    PB.registerLoopAnalyses(LAM);
+    PB.crossRegisterProxies(LAM, FAM, CGAM, MAM);
+    llvm::ModulePassManager MPM = PB.buildPerModuleDefaultPipeline(llvm::OptimizationLevel::O3);
+    MPM.run(m, MAM);
+
+#else
     llvm::legacy::PassManager mpm;
     mpm.add(new llvm::TargetLibraryInfoWrapperPass(TM->getTargetTriple()));
     mpm.add(llvm::createTargetTransformInfoWrapperPass(TM->getTargetIRAnalysis()));
     llvm::legacy::FunctionPassManager fpm(&m);
     fpm.add(llvm::createTargetTransformInfoWrapperPass(TM->getTargetIRAnalysis()));
-
-#if LLVM_VERSION_MAJOR >= 17
-    // TODO: https://llvm.org/docs/NewPassManager.html
-#else
     int optLevel = 3;
     int sizeLevel = 0;
     llvm::PassManagerBuilder builder;
@@ -413,16 +442,14 @@ void LLVMEvaluator::opt(llvm::Module &m) {
     builder.SLPVectorize = true;
     builder.populateFunctionPassManager(fpm);
     builder.populateModulePassManager(mpm);
-#endif
-
     fpm.doInitialization();
     for (llvm::Function &func : m) {
         fpm.run(func);
     }
     fpm.doFinalization();
-
     mpm.add(llvm::createVerifierPass());
     mpm.run(m);
+#endif
 }
 
 std::string LLVMEvaluator::module_to_string(llvm::Module &m) {
@@ -438,11 +465,20 @@ void LLVMEvaluator::print_version_message()
     llvm::cl::PrintVersionMessage();
 }
 
+std::string LLVMEvaluator::llvm_version()
+{
+    return LLVM_VERSION_STRING;
+}
+
 llvm::LLVMContext &LLVMEvaluator::get_context()
 {
     return *context;
 }
 
+const llvm::DataLayout &LLVMEvaluator::get_jit_data_layout() {
+    return jit->getDataLayout();
+}
+
 void LLVMEvaluator::print_targets()
 {
     llvm::InitializeNativeTarget();
