@@ -1,8 +1,6 @@
 #ifndef LFORTRAN_LLVM_UTILS_H
 #define LFORTRAN_LLVM_UTILS_H
 
-#include <memory>
-
 #include <llvm/IR/Value.h>
 #include <llvm/IR/Module.h>
 #include <llvm/IR/IRBuilder.h>
@@ -85,8 +83,9 @@ namespace LCompilers {
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
                     llvm::Type::getInt8Ty(context)->getPointerTo(),
-                    {llvm::Type::getInt32Ty(context),
-                    llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
+                    {llvm::Type::getInt8Ty(context)->getPointerTo(),
+                    llvm::Type::getInt8Ty(context)->getPointerTo(),
+                    llvm::Type::getInt32Ty(context)}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lcompilers_string_format_fortran", &module);
         }
@@ -94,22 +93,51 @@ namespace LCompilers {
     }
 
     static inline llvm::Value* lfortran_str_copy(llvm::Value* dest, llvm::Value *src, bool is_allocatable,
-        llvm::Module &module, llvm::IRBuilder<> &builder, llvm::LLVMContext &context) {
-        std::string runtime_func_name = "_lfortran_strcpy";
-        llvm::Function *fn = module.getFunction(runtime_func_name);
-        if (!fn) {
-            llvm::FunctionType *function_type = llvm::FunctionType::get(
-                     llvm::Type::getVoidTy(context), {
-                        llvm::Type::getInt8Ty(context)->getPointerTo()->getPointerTo(),
-                        llvm::Type::getInt8Ty(context)->getPointerTo(),
-                        llvm::Type::getInt8Ty(context)
-                    }, false);
-            fn = llvm::Function::Create(function_type,
-                    llvm::Function::ExternalLinkage, runtime_func_name, module);
+        llvm::Module &module, llvm::IRBuilder<> &builder, llvm::LLVMContext &context, llvm::Type* string_descriptor ) {
+        if(!is_allocatable){
+            std::string runtime_func_name = "_lfortran_strcpy_pointer_string";
+            llvm::Function *fn = module.getFunction(runtime_func_name);
+            if (!fn) {
+                llvm::FunctionType *function_type = llvm::FunctionType::get(
+                        llvm::Type::getVoidTy(context),
+                        {
+                            llvm::Type::getInt8Ty(context)->getPointerTo()->getPointerTo(),
+                            llvm::Type::getInt8Ty(context)->getPointerTo()
+                        }, false);
+                fn = llvm::Function::Create(function_type,
+                        llvm::Function::ExternalLinkage, runtime_func_name, module);
+            }
+            return builder.CreateCall(fn, {dest, src});
+        } else {
+            std::string runtime_func_name = "_lfortran_strcpy_descriptor_string";
+            llvm::Value *src_char_ptr, *dest_char_ptr, *string_size, *string_capacity;
+            std::vector<llvm::Value*> idx {
+                llvm::ConstantInt::get(context, llvm::APInt(32, 0)),
+                llvm::ConstantInt::get(context, llvm::APInt(32, 0))};
+            // Fetch char* from `src` and `dest` + Fetch string_size, string_capacity from `dest`
+            dest_char_ptr = builder.CreateGEP(string_descriptor, dest, idx);
+            src_char_ptr = builder.CreateLoad(llvm::Type::getInt8Ty(context)->getPointerTo(),
+                builder.CreateGEP(string_descriptor, src, idx));
+            idx[1] = llvm::ConstantInt::get(context, llvm::APInt(32, 1));
+            string_size = builder.CreateGEP(string_descriptor, dest, idx);
+            idx[1] = llvm::ConstantInt::get(context, llvm::APInt(32, 2));
+            string_capacity =  builder.CreateGEP(string_descriptor, dest, idx);
+            llvm::Function *fn = module.getFunction(runtime_func_name);
+            if (!fn) {
+                llvm::FunctionType *function_type = llvm::FunctionType::get(
+                        llvm::Type::getVoidTy(context),
+                        {
+                            llvm::Type::getInt8Ty(context)->getPointerTo()->getPointerTo(),
+                            llvm::Type::getInt8Ty(context)->getPointerTo(),
+                            llvm::Type::getInt64Ty(context)->getPointerTo(),
+                            llvm::Type::getInt64Ty(context)->getPointerTo()
+                        }, false);
+                fn = llvm::Function::Create(function_type,
+                        llvm::Function::ExternalLinkage, runtime_func_name, module);
+            }
+            return builder.CreateCall(fn, {dest_char_ptr, src_char_ptr, string_size, string_capacity});
         }
-        llvm::Value* free_string = llvm::ConstantInt::get(
-            llvm::Type::getInt8Ty(context), llvm::APInt(8, is_allocatable));
-        return builder.CreateCall(fn, {dest, src, free_string});
+
     }
 
     static inline void print_error(llvm::LLVMContext &context, llvm::Module &module,
@@ -225,6 +253,7 @@ namespace LCompilers {
             llvm::StructType *complex_type_4, *complex_type_8;
             llvm::StructType *complex_type_4_ptr, *complex_type_8_ptr;
             llvm::PointerType *character_type;
+            llvm::Type* string_descriptor;
 
             LLVMUtils(llvm::LLVMContext& context,
                 llvm::IRBuilder<>* _builder, std::string& der_type_name_,
@@ -239,31 +268,39 @@ namespace LCompilers {
                 std::map<llvm::Value *, llvm::Type *> &ptr_type_);
 
             llvm::Value* create_gep(llvm::Value* ds, int idx);
-
-            llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, int idx);
-
             llvm::Value* create_gep(llvm::Value* ds, llvm::Value* idx);
 
             llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, llvm::Value* idx);
+            llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, int idx);
+            llvm::Value* create_gep2(ASR::ttype_t *t, llvm::Value* ds, int idx);
+            llvm::Value* create_gep2(ASR::ttype_t *t, llvm::Value* ds, llvm::Value* idx);
 
             llvm::Value* create_ptr_gep(llvm::Value* ptr, int idx);
-
-            llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, int idx);
-
             llvm::Value* create_ptr_gep(llvm::Value* ptr, llvm::Value* idx);
 
+            llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, int idx);
             llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, llvm::Value* idx);
+            llvm::Value* create_ptr_gep2(ASR::ttype_t* type, llvm::Value* ptr, int idx);
+            llvm::Value* create_ptr_gep2(ASR::ttype_t* type, llvm::Value* ptr, llvm::Value* idx);
 
             llvm::Value* CreateLoad(llvm::Value *x);
+
             llvm::Value* CreateLoad2(llvm::Type *t, llvm::Value *x);
             llvm::Value* CreateLoad2(ASR::ttype_t *type, llvm::Value *x);
+
             llvm::Value* CreateGEP(llvm::Value *x, std::vector<llvm::Value *> &idx);
             llvm::Value* CreateGEP2(llvm::Type *t, llvm::Value *x,
                 std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateGEP2(llvm::Type *type, llvm::Value *x, int idx);
             llvm::Value* CreateGEP2(ASR::ttype_t *type, llvm::Value *x, int idx);
+            llvm::Value* CreateGEP2(ASR::ttype_t *t, llvm::Value *x,
+                std::vector<llvm::Value *> &idx);
+
             llvm::Value* CreateInBoundsGEP(llvm::Value *x, std::vector<llvm::Value *> &idx);
             llvm::Value* CreateInBoundsGEP2(llvm::Type *t, llvm::Value *x,
                 std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateInBoundsGEP2(ASR::ttype_t *t, llvm::Value *x,
+                std::vector<llvm::Value *> &idx);
 
             llvm::AllocaInst* CreateAlloca(llvm::Type* type,
                 llvm::Value* size=nullptr, std::string Name="",
@@ -279,11 +316,30 @@ namespace LCompilers {
             llvm::Value* lfortran_str_cmp(llvm::Value* left_arg, llvm::Value* right_arg,
                                           std::string runtime_func_name, llvm::Module& module);
 
+            /*
+             * Initialize string with empty characters.
+            */
+            void string_init(llvm::Value* arg_size, llvm::Value* arg_string);
+
+            /*
+             * Allocate heap memory for string.
+             * Fill string with empty characters.
+            */
+            void initialize_string_heap(llvm::Value* str, llvm::Value* len);
+            void initialize_string_heap(llvm::Value* str, int64_t len);
+
+            /*
+             * Allocate stack memory for string.
+             * Fill string with empty characters.
+            */
+            void initialize_string_stack(llvm::Value* str, llvm::Value* len){(void)str;(void)len;throw LCompilersException("Not Implemented Yet");};
+            void initialize_string_stack(llvm::Value* str, int64_t len){(void)str;(void)len;throw LCompilersException("Not Implemented Yet");};
+
             llvm::Value* is_equal_by_value(llvm::Value* left, llvm::Value* right,
-                                           llvm::Module& module, ASR::ttype_t* asr_type);
+                                           llvm::Module* module, ASR::ttype_t* asr_type);
 
             llvm::Value* is_ineq_by_value(llvm::Value* left, llvm::Value* right,
-                                          llvm::Module& module, ASR::ttype_t* asr_type,
+                                          llvm::Module* module, ASR::ttype_t* asr_type,
                                           int8_t overload_id, ASR::ttype_t* int32_type=nullptr);
 
             void set_module(llvm::Module* module_);
@@ -297,10 +353,10 @@ namespace LCompilers {
 
             llvm::Type* getStructType(ASR::ttype_t* _type, llvm::Module* module, bool is_pointer=false);
 
-            llvm::Type* getUnionType(ASR::UnionType_t* union_type,
+            llvm::Type* getUnion(ASR::Union_t* union_type,
                 llvm::Module* module, bool is_pointer=false);
 
-            llvm::Type* getUnionType(ASR::ttype_t* _type,
+            llvm::Type* getUnion(ASR::ttype_t* _type,
                 llvm::Module* module, bool is_pointer=false);
 
             llvm::Type* getClassType(ASR::Class_t* der_type, bool is_pointer=false);
@@ -350,6 +406,8 @@ namespace LCompilers {
                 ASR::ttype_t* asr_type, llvm::Module* module,
                 std::map<std::string, std::map<std::string, int>>& name2memidx);
 
+            llvm::Value* convert_kind(llvm::Value* val, llvm::Type* target_type);
+
 
             // Note: `llvm_utils->create_if_else` and `create_loop` are optional APIs
             // that do not have to be used. Many times, for more complicated
@@ -405,11 +463,11 @@ namespace LCompilers {
                                         int32_t type_size);
 
             void list_init(std::string& type_code, llvm::Value* list,
-                           llvm::Module& module, llvm::Value* initial_capacity,
+                           llvm::Module* module, llvm::Value* initial_capacity,
                            llvm::Value* n);
 
             void list_init(std::string& type_code, llvm::Value* list,
-                            llvm::Module& module, int32_t initial_capacity=1,
+                            llvm::Module* module, int32_t initial_capacity=1,
                             int32_t n=0);
 
             llvm::Value* get_pointer_to_list_data(llvm::Value* list);
@@ -428,12 +486,12 @@ namespace LCompilers {
 
             llvm::Value* read_item(llvm::Value* list, llvm::Value* pos,
                                    bool enable_bounds_checking,
-                                   llvm::Module& module, bool get_pointer=false);
+                                   llvm::Module* module, bool get_pointer=false);
 
             llvm::Value* len(llvm::Value* list);
 
             void check_index_within_bounds(llvm::Value* list, llvm::Value* pos,
-                                           llvm::Module& module);
+                                           llvm::Module* module);
 
             void write_item(llvm::Value* list, llvm::Value* pos,
                 llvm::Value* item, ASR::ttype_t* asr_type,
@@ -442,7 +500,7 @@ namespace LCompilers {
 
             void write_item(llvm::Value* list, llvm::Value* pos,
                             llvm::Value* item, bool enable_bounds_checking,
-                            llvm::Module& module);
+                            llvm::Module* module);
 
             void append(llvm::Value* list, llvm::Value* item,
                         ASR::ttype_t* asr_type, llvm::Module* module,
@@ -457,38 +515,38 @@ namespace LCompilers {
                          ASR::ttype_t* asr_type, llvm::Module* module);
 
             void remove(llvm::Value* list, llvm::Value* item,
-                        ASR::ttype_t* item_type, llvm::Module& module);
+                        ASR::ttype_t* item_type, llvm::Module* module);
 
             llvm::Value* pop_position(llvm::Value* list, llvm::Value* pos,
                                       ASR::ttype_t* list_type, llvm::Module* module,
                                       std::map<std::string, std::map<std::string, int>>& name2memidx);
 
-            llvm::Value* pop_last(llvm::Value* list, ASR::ttype_t* list_type, llvm::Module& module);
+            llvm::Value* pop_last(llvm::Value* list, ASR::ttype_t* list_type, llvm::Module* module);
 
             void list_clear(llvm::Value* list);
 
-            void reverse(llvm::Value* list, llvm::Module& module);
+            void reverse(llvm::Value* list, llvm::Module* module);
 
             llvm::Value* find_item_position(llvm::Value* list,
                 llvm::Value* item, ASR::ttype_t* item_type,
-                llvm::Module& module, llvm::Value* start=nullptr,
+                llvm::Module* module, llvm::Value* start=nullptr,
                 llvm::Value* end=nullptr);
 
             llvm::Value* index(llvm::Value* list, llvm::Value* item,
                                 llvm::Value* start, llvm::Value* end,
-                                ASR::ttype_t* item_type, llvm::Module& module);
+                                ASR::ttype_t* item_type, llvm::Module* module);
 
             llvm::Value* count(llvm::Value* list, llvm::Value* item,
-                                ASR::ttype_t* item_type, llvm::Module& module);
+                                ASR::ttype_t* item_type, llvm::Module* module);
 
-            void free_data(llvm::Value* list, llvm::Module& module);
+            void free_data(llvm::Value* list, llvm::Module* module);
 
             llvm::Value* check_list_equality(llvm::Value* l1, llvm::Value* l2, ASR::ttype_t *item_type,
-                llvm::LLVMContext& context, llvm::IRBuilder<>* builder, llvm::Module& module);
+                llvm::LLVMContext& context, llvm::IRBuilder<>* builder, llvm::Module* module);
 
             llvm::Value* check_list_inequality(llvm::Value* l1, llvm::Value* l2,
                 ASR::ttype_t *item_type, llvm::LLVMContext& context,
-                llvm::IRBuilder<>* builder, llvm::Module& module,
+                llvm::IRBuilder<>* builder, llvm::Module* module,
                 int8_t overload_id, ASR::ttype_t* int32_type=nullptr);
 
             void list_repeat_copy(llvm::Value* repeat_list, llvm::Value* init_list,
@@ -530,15 +588,15 @@ namespace LCompilers {
 
             llvm::Value* check_tuple_equality(llvm::Value* t1, llvm::Value* t2,
                 ASR::Tuple_t* tuple_type, llvm::LLVMContext& context,
-                llvm::IRBuilder<>* builder, llvm::Module& module);
+                llvm::IRBuilder<>* builder, llvm::Module* module);
 
             llvm::Value* check_tuple_inequality(llvm::Value* t1, llvm::Value* t2,
                 ASR::Tuple_t* tuple_type, llvm::LLVMContext& context,
-                llvm::IRBuilder<>* builder, llvm::Module& module, int8_t overload_id);
+                llvm::IRBuilder<>* builder, llvm::Module* module, int8_t overload_id);
 
             void concat(llvm::Value* t1, llvm::Value* t2, ASR::Tuple_t* tuple_type_1,
                         ASR::Tuple_t* tuple_type_2, llvm::Value* concat_tuple,
-                        ASR::Tuple_t* concat_tuple_type, llvm::Module& module,
+                        ASR::Tuple_t* concat_tuple_type, llvm::Module* module,
                         std::map<std::string, std::map<std::string, int>>& name2memidx);
     };
 
@@ -594,7 +652,7 @@ namespace LCompilers {
 
             virtual
             llvm::Value* get_key_hash(llvm::Value* capacity, llvm::Value* key,
-                ASR::ttype_t* key_asr_type, llvm::Module& module);
+                ASR::ttype_t* key_asr_type, llvm::Module* module);
 
             virtual
             void resolve_collision_for_write(llvm::Value* dict, llvm::Value* key_hash,
@@ -605,17 +663,17 @@ namespace LCompilers {
 
             virtual
             llvm::Value* resolve_collision_for_read(llvm::Value* dict, llvm::Value* key_hash,
-                llvm::Value* key, llvm::Module& module,
+                llvm::Value* key, llvm::Module* module,
                 ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type) = 0;
 
             virtual
             llvm::Value* resolve_collision_for_read_with_bound_check(llvm::Value* dict, llvm::Value* key_hash,
-                llvm::Value* key, llvm::Module& module,
+                llvm::Value* key, llvm::Module* module,
                 ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type) = 0;
 
             virtual
             llvm::Value* resolve_collision_for_read_with_default(llvm::Value* dict, llvm::Value* key_hash,
-                llvm::Value* key, llvm::Module& module,
+                llvm::Value* key, llvm::Module* module,
                 ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type, llvm::Value* def_value) = 0;
 
             virtual
@@ -638,17 +696,17 @@ namespace LCompilers {
 
             virtual
             llvm::Value* read_item(llvm::Value* dict, llvm::Value* key,
-                llvm::Module& module, ASR::Dict_t* dict_type, bool enable_bounds_checking,
+                llvm::Module* module, ASR::Dict_t* dict_type, bool enable_bounds_checking,
                 bool get_pointer=false) = 0;
 
             virtual
             llvm::Value* get_item(llvm::Value* dict, llvm::Value* key,
-                llvm::Module& module, ASR::Dict_t* dict_type, llvm::Value* def_value,
+                llvm::Module* module, ASR::Dict_t* dict_type, llvm::Value* def_value,
                 bool get_pointer=false) = 0;
 
             virtual
             llvm::Value* pop_item(llvm::Value* dict, llvm::Value* key,
-                llvm::Module& module, ASR::Dict_t* dict_type,
+                llvm::Module* module, ASR::Dict_t* dict_type,
                 bool get_pointer=false) = 0;
 
 
@@ -669,7 +727,7 @@ namespace LCompilers {
             virtual
             void get_elements_list(llvm::Value* dict,
                 llvm::Value* elements_list, ASR::ttype_t* key_asr_type,
-                ASR::ttype_t* value_asr_type, llvm::Module& module,
+                ASR::ttype_t* value_asr_type, llvm::Module* module,
                 std::map<std::string, std::map<std::string, int>>& name2memidx,
                 bool key_or_value) = 0;
 
@@ -703,7 +761,7 @@ namespace LCompilers {
             virtual
             void resolve_collision(llvm::Value* capacity, llvm::Value* key_hash,
                                 llvm::Value* key, llvm::Value* key_list,
-                                llvm::Value* key_mask, llvm::Module& module,
+                                llvm::Value* key_mask, llvm::Module* module,
                                 ASR::ttype_t* key_asr_type, bool for_read=false);
 
             void resolve_collision_for_write(llvm::Value* dict, llvm::Value* key_hash,
@@ -712,20 +770,20 @@ namespace LCompilers {
                                           ASR::ttype_t* value_asr_type,
                                           std::map<std::string, std::map<std::string, int>>& name2memidx);
 
-            void _check_key_present_or_default(llvm::Module& module, llvm::Value *key, llvm::Value *key_list,
-                ASR::ttype_t* key_asr_type, llvm::Value *value_list, llvm::Value *pos,
+            void _check_key_present_or_default(llvm::Module* module, llvm::Value *key, llvm::Value *key_list,
+                ASR::ttype_t* key_asr_type, llvm::Value *value_list, ASR::ttype_t* value_asr_type, llvm::Value *pos,
                 llvm::Value *def_value, llvm::Value* &result);
 
             llvm::Value* resolve_collision_for_read(llvm::Value* dict, llvm::Value* key_hash,
-                                                 llvm::Value* key, llvm::Module& module,
+                                                 llvm::Value* key, llvm::Module* module,
                                                  ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type);
 
             llvm::Value* resolve_collision_for_read_with_bound_check(llvm::Value* dict, llvm::Value* key_hash,
-                                                 llvm::Value* key, llvm::Module& module,
+                                                 llvm::Value* key, llvm::Module* module,
                                                  ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type);
 
             llvm::Value* resolve_collision_for_read_with_default(llvm::Value* dict, llvm::Value* key_hash,
-                                                 llvm::Value* key, llvm::Module& module,
+                                                 llvm::Value* key, llvm::Module* module,
                                                  ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type,
                                                  llvm::Value* def_value);
 
@@ -740,15 +798,15 @@ namespace LCompilers {
                                               std::map<std::string, std::map<std::string, int>>& name2memidx);
 
             llvm::Value* read_item(llvm::Value* dict, llvm::Value* key,
-                                   llvm::Module& module, ASR::Dict_t* key_asr_type, bool enable_bounds_checking,
+                                   llvm::Module* module, ASR::Dict_t* key_asr_type, bool enable_bounds_checking,
                                    bool get_pointer=false);
 
             llvm::Value* get_item(llvm::Value* dict, llvm::Value* key,
-                                   llvm::Module& module, ASR::Dict_t* key_asr_type, llvm::Value* def_value,
+                                   llvm::Module* module, ASR::Dict_t* key_asr_type, llvm::Value* def_value,
                                    bool get_pointer=false);
 
             llvm::Value* pop_item(llvm::Value* dict, llvm::Value* key,
-                                   llvm::Module& module, ASR::Dict_t* dict_type,
+                                   llvm::Module* module, ASR::Dict_t* dict_type,
                                    bool get_pointer=false);
 
             virtual
@@ -762,7 +820,7 @@ namespace LCompilers {
 
             void get_elements_list(llvm::Value* dict,
                 llvm::Value* elements_list, ASR::ttype_t* key_asr_type,
-                ASR::ttype_t* value_asr_type, llvm::Module& module,
+                ASR::ttype_t* value_asr_type, llvm::Module* module,
                 std::map<std::string, std::map<std::string, int>>& name2memidx,
                 bool key_or_value);
 
@@ -779,7 +837,7 @@ namespace LCompilers {
 
             void resolve_collision(llvm::Value* capacity, llvm::Value* key_hash,
                                 llvm::Value* key, llvm::Value* key_list,
-                                llvm::Value* key_mask, llvm::Module& module,
+                                llvm::Value* key_mask, llvm::Module* module,
                                 ASR::ttype_t* key_asr_type, bool for_read=false);
 
             void resolve_collision_for_write(llvm::Value* dict, llvm::Value* key_hash,
@@ -789,15 +847,15 @@ namespace LCompilers {
                                             std::map<std::string, std::map<std::string, int>>& name2memidx);
 
             llvm::Value* resolve_collision_for_read(llvm::Value* dict, llvm::Value* key_hash,
-                                                    llvm::Value* key, llvm::Module& module,
+                                                    llvm::Value* key, llvm::Module* module,
                                                     ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type);
 
             llvm::Value* resolve_collision_for_read_with_bound_check(llvm::Value* dict, llvm::Value* key_hash,
-                                                    llvm::Value* key, llvm::Module& module,
+                                                    llvm::Value* key, llvm::Module* module,
                                                     ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type);
 
             llvm::Value* resolve_collision_for_read_with_default(llvm::Value* dict, llvm::Value* key_hash,
-                                                    llvm::Value* key, llvm::Module& module,
+                                                    llvm::Value* key, llvm::Module* module,
                                                     ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type,
                                                     llvm::Value *def_value);
 
@@ -828,7 +886,7 @@ namespace LCompilers {
             void resolve_collision(llvm::Value* capacity, llvm::Value* key_hash,
                 llvm::Value* key, llvm::Value* key_value_pair_linked_list,
                 llvm::Type* kv_pair_type, llvm::Value* key_mask,
-                llvm::Module& module, ASR::ttype_t* key_asr_type);
+                llvm::Module* module, ASR::ttype_t* key_asr_type);
 
             llvm::Type* get_key_value_pair_type(std::string key_type_code, std::string value_type_code);
 
@@ -866,15 +924,15 @@ namespace LCompilers {
                 std::map<std::string, std::map<std::string, int>>& name2memidx);
 
             llvm::Value* resolve_collision_for_read(llvm::Value* dict, llvm::Value* key_hash,
-                llvm::Value* key, llvm::Module& module,
+                llvm::Value* key, llvm::Module* module,
                 ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type);
 
             llvm::Value* resolve_collision_for_read_with_bound_check(llvm::Value* dict, llvm::Value* key_hash,
-                llvm::Value* key, llvm::Module& module,
+                llvm::Value* key, llvm::Module* module,
                 ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type);
 
             llvm::Value* resolve_collision_for_read_with_default(llvm::Value* dict, llvm::Value* key_hash,
-                llvm::Value* key, llvm::Module& module,
+                llvm::Value* key, llvm::Module* module,
                 ASR::ttype_t* key_asr_type, ASR::ttype_t* value_asr_type,
                 llvm::Value* def_value);
 
@@ -889,15 +947,15 @@ namespace LCompilers {
                 std::map<std::string, std::map<std::string, int>>& name2memidx);
 
             llvm::Value* read_item(llvm::Value* dict, llvm::Value* key,
-                llvm::Module& module, ASR::Dict_t* dict_type, bool enable_bounds_checking,
+                llvm::Module* module, ASR::Dict_t* dict_type, bool enable_bounds_checking,
                 bool get_pointer=false);
 
             llvm::Value* get_item(llvm::Value* dict, llvm::Value* key,
-                llvm::Module& module, ASR::Dict_t* dict_type, llvm::Value* def_value,
+                llvm::Module* module, ASR::Dict_t* dict_type, llvm::Value* def_value,
                 bool get_pointer=false);
 
             llvm::Value* pop_item(llvm::Value* dict, llvm::Value* key,
-                llvm::Module& module, ASR::Dict_t* dict_type,
+                llvm::Module* module, ASR::Dict_t* dict_type,
                 bool get_pointer=false);
 
             llvm::Value* get_pointer_to_keymask(llvm::Value* dict);
@@ -910,7 +968,7 @@ namespace LCompilers {
 
             void get_elements_list(llvm::Value* dict,
                 llvm::Value* elements_list, ASR::ttype_t* key_asr_type,
-                ASR::ttype_t* value_asr_type, llvm::Module& module,
+                ASR::ttype_t* value_asr_type, llvm::Module* module,
                 std::map<std::string, std::map<std::string, int>>& name2memidx,
                 bool key_or_value);
 
@@ -962,7 +1020,7 @@ namespace LCompilers {
             llvm::Value* get_pointer_to_capacity(llvm::Value* set) = 0;
 
             llvm::Value* get_el_hash(llvm::Value* capacity, llvm::Value* el,
-                ASR::ttype_t* el_asr_type, llvm::Module& module);
+                ASR::ttype_t* el_asr_type, llvm::Module* module);
 
             virtual
             void resolve_collision_for_write(
@@ -989,12 +1047,12 @@ namespace LCompilers {
             virtual
             void resolve_collision_for_read_with_bound_check(
                 llvm::Value* set, llvm::Value* el_hash, llvm::Value* el,
-                llvm::Module& module, ASR::ttype_t* el_asr_type) = 0;
+                llvm::Module* module, ASR::ttype_t* el_asr_type) = 0;
 
             virtual
             void remove_item(
                 llvm::Value* set, llvm::Value* el,
-                llvm::Module& module, ASR::ttype_t* el_asr_type) = 0;
+                llvm::Module* module, ASR::ttype_t* el_asr_type) = 0;
 
             virtual
             void set_deepcopy(
@@ -1042,7 +1100,7 @@ namespace LCompilers {
             void resolve_collision(
                 llvm::Value* capacity, llvm::Value* el_hash,
                 llvm::Value* el, llvm::Value* el_list,
-                llvm::Value* el_mask, llvm::Module& module,
+                llvm::Value* el_mask, llvm::Module* module,
                 ASR::ttype_t* el_asr_type, bool for_read=false);
 
             void resolve_collision_for_write(
@@ -1060,11 +1118,11 @@ namespace LCompilers {
 
             void resolve_collision_for_read_with_bound_check(
                 llvm::Value* set, llvm::Value* el_hash, llvm::Value* el,
-                llvm::Module& module, ASR::ttype_t* el_asr_type);
+                llvm::Module* module, ASR::ttype_t* el_asr_type);
 
             void remove_item(
                 llvm::Value* set, llvm::Value* el,
-                llvm::Module& module, ASR::ttype_t* el_asr_type);
+                llvm::Module* module, ASR::ttype_t* el_asr_type);
 
             void set_deepcopy(
                 llvm::Value* src, llvm::Value* dest,
@@ -1092,7 +1150,7 @@ namespace LCompilers {
             void resolve_collision(
                 llvm::Value* el_hash, llvm::Value* el, llvm::Value* el_linked_list,
                 llvm::Type* el_struct_type, llvm::Value* el_mask,
-                llvm::Module& module, ASR::ttype_t* el_asr_type);
+                llvm::Module* module, ASR::ttype_t* el_asr_type);
 
             void write_el_linked_list(
                 llvm::Value* el_ll, llvm::Value* set, llvm::Value* capacity,
@@ -1141,11 +1199,11 @@ namespace LCompilers {
 
             void resolve_collision_for_read_with_bound_check(
                 llvm::Value* set, llvm::Value* el_hash, llvm::Value* el,
-                llvm::Module& module, ASR::ttype_t* el_asr_type);
+                llvm::Module* module, ASR::ttype_t* el_asr_type);
 
             void remove_item(
                 llvm::Value* set, llvm::Value* el,
-                llvm::Module& module, ASR::ttype_t* el_asr_type);
+                llvm::Module* module, ASR::ttype_t* el_asr_type);
 
             void set_deepcopy(
                 llvm::Value* src, llvm::Value* dest,
