@@ -263,12 +263,6 @@ intrinsic_funcs_args = {
             "return": "int32"
         },
     ],
-    "IsContiguous": [
-        {
-            "args": [("any",)],
-            "return": "logical"
-        },
-    ],
     "OutOfRange": [
         {
             "args": [("int", "real", "bool"), ("real", "real", "bool"), ("int", "int", "bool"), ("real", "int", "bool")],
@@ -854,9 +848,9 @@ intrinsic_funcs_args = {
         {
             "args": [("int",), ("real",), ("complex",)],
             "return": "real32",
-            "kind_arg": True, 
+            "kind_arg": True,
             "real_32_except_complex": True
-        },   
+        },
     ],
     "Int": [
         {
@@ -1051,11 +1045,7 @@ def add_create_func_return_src(func_name):
             src += indent * 2 + f"set_kind_to_ttype_t(type_,upper_kind);\n"
         else:
             src += indent * 2 + "ASR::ttype_t* type_ = nullptr;\n"
-            src += indent * 2 + "if (use_experimental_simplifier) {\n"
-            src += indent * 2 +     f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
-            src += indent * 2 + "} else {\n"
-            src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(expr_type(args[{ret_type_arg_idx}]));\n"
-            src += indent * 2 + "}\n"
+            src += indent * 2 + f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
         ret_type = "type_"
     kind_arg = arg_infos[0].get("kind_arg", False)
     src += indent * 2 + f"ASR::ttype_t *return_type = {ret_type};\n"
@@ -1092,27 +1082,17 @@ def add_create_func_return_src(func_name):
             "ASRUtils::expr_type(m_args[0]), m_args[0], return_type, m_value);\n"
 
     else:
-        src += indent * 2 + "if (use_experimental_simplifier) {\n"
-        src += indent * 3 +     f"for( size_t i = 0; i < {no_of_args}; i++ ) " + "{\n"
-        src += indent * 4 +         "ASR::ttype_t* type = ASRUtils::expr_type(args[i]);\n"
-        src += indent * 4 +         "if (ASRUtils::is_array(type)) {\n"
-        src += indent * 5 +             "ASR::dimension_t* m_dims = nullptr;\n"
-        src += indent * 5 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
-        src += indent * 5 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
+        src += indent * 2 +     f"for( size_t i = 0; i < {no_of_args}; i++ ) " + "{\n"
+        src += indent * 3 +         "ASR::ttype_t* type = ASRUtils::expr_type(args[i]);\n"
+        src += indent * 3 +         "if (ASRUtils::is_array(type)) {\n"
+        src += indent * 4 +             "ASR::dimension_t* m_dims = nullptr;\n"
+        src += indent * 4 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
+        src += indent * 4 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
         src +=                              "return_type, m_dims, n_dims, ASR::abiType::Source, false, "
         src +=                              "ASR::array_physical_typeType::DescriptorArray);\n"
-        src += indent * 5 +             "break;\n"
-        src += indent * 4 +         "}\n"
-        src += indent * 3 +     "}\n"
-        src += indent * 2 + "}\n"
-        if ret_type_val:
-            src += indent * 2 + "else {\n"
-            src += indent * 3 +     "ASR::ttype_t* type = ASRUtils::expr_type(args[0]);\n"
-            src += indent * 3 +     "if (ASR::is_a<ASR::Array_t>(*type)) {\n"
-            src += indent * 4 +         "ASR::Array_t* e = ASR::down_cast<ASR::Array_t>(type);\n"
-            src += indent * 4 +         f"return_type = TYPE(ASR::make_Array_t(al, type->base.loc,  return_type, e->m_dims, e->n_dims, ASR::array_physical_typeType::FixedSizeArray));\n"
-            src += indent * 3 +     "}\n"
-            src += indent * 2 + "}\n"
+        src += indent * 4 +             "break;\n"
+        src += indent * 3 +         "}\n"
+        src += indent * 2 +     "}\n"
 
         src += indent * 2 + "if (all_args_evaluated(m_args)) {\n"
         src += indent * 3 +     f"Vec<ASR::expr_t*> args_values; args_values.reserve(al, {no_of_args});\n"
