@@ -7,7 +7,6 @@
 #include <libasr/pass/pass_utils.h>
 
 #include <vector>
-#include <utility>
 
 namespace LCompilers {
 
@@ -59,18 +58,11 @@ class ReplaceInitExpr: public ASR::BaseExprReplacer<ReplaceInitExpr> {
             symtab2decls[current_scope] = result_vec_;
         }
         Vec<ASR::stmt_t*>* result_vec = &symtab2decls[current_scope];
-        bool remove_original_statement = false;
         if( casted_type != nullptr ) {
             casted_type = ASRUtils::type_get_past_array(casted_type);
         }
-        if (ASRUtils::use_experimental_simplifier) {
-            PassUtils::ReplacerUtils::replace_ArrayConstructor_(al, x, result_var, result_vec,
-                current_scope, perform_cast, cast_kind, casted_type);
-        } else {
-            PassUtils::ReplacerUtils::replace_ArrayConstructor(x, this,
-                remove_original_statement, result_vec,
-                perform_cast, cast_kind, casted_type);
-        }
+        PassUtils::ReplacerUtils::replace_ArrayConstructor_(al, x, result_var, result_vec,
+            current_scope, perform_cast, cast_kind, casted_type);
         *current_expr = nullptr;
     }
 
@@ -192,7 +184,7 @@ class InitExprVisitor : public ASR::CallReplacerOnExpressionsVisitor<InitExprVis
                  (ASR::is_a<ASR::Module_t>(*asr_owner) &&
                   (ASR::is_a<ASR::ArrayConstant_t>(*symbolic_value) ||
                   ASR::is_a<ASR::ArrayConstructor_t>(*symbolic_value))) ||
-                (x.m_storage == ASR::storage_typeType::Save && 
+                (x.m_storage == ASR::storage_typeType::Save &&
                 ASR::is_a<ASR::Function_t>(*ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner)))) {
                 return ;
             }
@@ -203,7 +195,7 @@ class InitExprVisitor : public ASR::CallReplacerOnExpressionsVisitor<InitExprVis
 
             SymbolTable* current_scope_copy = current_scope;
             current_scope = x.m_parent_symtab;
-            if (x.m_symbolic_value) {
+            if (x.m_symbolic_value && (x.m_storage != ASR::storage_typeType::Parameter)) {
                 ASR::expr_t** current_expr_copy = current_expr;
                 current_expr = const_cast<ASR::expr_t**>(&(x.m_symbolic_value));
                 call_replacer();
