@@ -68,7 +68,7 @@ public:
             return current_scope->get_symbol(new_sym_name);
         }
 
-        ASR::symbol_t* new_v = ASR::down_cast<ASR::symbol_t>(ASR::make_Variable_t(
+        ASR::symbol_t* new_v = ASR::down_cast<ASR::symbol_t>(ASRUtils::make_Variable_t_util(
             al, x->base.base.loc,
             current_scope, s2c(al, new_sym_name), x->m_dependencies,
             x->n_dependencies, x->m_intent, x->m_symbolic_value,
@@ -140,7 +140,7 @@ public:
             t = ASRUtils::duplicate_type(al, type_subs[tp->m_param]);
         }
 
-        ASR::symbol_t* new_v = ASR::down_cast<ASR::symbol_t>(ASR::make_Variable_t(
+        ASR::symbol_t* new_v = ASR::down_cast<ASR::symbol_t>(ASRUtils::make_Variable_t_util(
             al, x->base.base.loc, current_scope, x->m_name, x->m_dependencies,
             x->n_dependencies, x->m_intent, x->m_symbolic_value,
             x->m_value, x->m_storage, t, x->m_type_declaration,
@@ -333,7 +333,7 @@ public:
         ASR::asr_t *result = ASR::make_Struct_t(al, x->base.base.loc,
             current_scope, s2c(al, new_sym_name),
             nullptr, 0,
-            data_member_names.p, data_member_names.size(),
+            data_member_names.p, data_member_names.size(), nullptr, 0,
             x->m_abi, x->m_access, x->m_is_packed, x->m_is_abstract,
             nullptr, 0, m_alignment, nullptr);
 
@@ -390,7 +390,7 @@ public:
         variable_dependencies_vec.reserve(al, 1);
         ASRUtils::collect_variable_dependencies(al, variable_dependencies_vec, new_type);
 
-        ASR::symbol_t* s = ASR::down_cast<ASR::symbol_t>(ASR::make_Variable_t(al,
+        ASR::symbol_t* s = ASR::down_cast<ASR::symbol_t>(ASRUtils::make_Variable_t_util(al,
             x->base.base.loc, current_scope, s2c(al, x->m_name), variable_dependencies_vec.p,
             variable_dependencies_vec.size(), x->m_intent, nullptr, nullptr, x->m_storage,
             new_type, nullptr, x->m_abi, x->m_access, x->m_presence, x->m_value_attr));
@@ -510,7 +510,7 @@ public:
         ASR::expr_t *target = duplicate_expr(x->m_target);
         ASR::expr_t *value = duplicate_expr(x->m_value);
         ASR::stmt_t *overloaded = duplicate_stmt(x->m_overloaded);
-        return ASR::make_Assignment_t(al, x->base.base.loc, target, value, overloaded);
+        return ASRUtils::make_Assignment_t_util(al, x->base.base.loc, target, value, overloaded, false);
     }
 
     ASR::asr_t* duplicate_DoLoop(ASR::DoLoop_t *x) {
@@ -694,7 +694,7 @@ public:
                     std::string new_struct_name = context_map[struct_name];
                     ASR::symbol_t *sym = func_scope->resolve_symbol(new_struct_name);
                     return ASRUtils::TYPE(
-                        ASR::make_StructType_t(al, s->base.base.loc, sym));
+                        ASRUtils::make_StructType_t_util(al, s->base.base.loc, sym));
                 } else {
                     return ttype;
                 }
@@ -842,10 +842,10 @@ void report_check_restriction(std::map<std::string, ASR::ttype_t*> type_subs,
                 = ASR::down_cast<ASR::TypeParameter_t>(f_param);
             if (!ASRUtils::check_equal_type(type_subs[f_tp->m_param],
                                             arg_param)) {
-                std::string rtype = ASRUtils::type_to_str(type_subs[f_tp->m_param]);
+                std::string rtype = ASRUtils::type_to_str_fortran(type_subs[f_tp->m_param]);
                 std::string rvar = ASRUtils::symbol_name(
                                     ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v);
-                std::string atype = ASRUtils::type_to_str(arg_param);
+                std::string atype = ASRUtils::type_to_str_fortran(arg_param);
                 std::string avar = ASRUtils::symbol_name(
                                     ASR::down_cast<ASR::Var_t>(arg->m_args[i])->m_v);
                 diagnostics.add(diag::Diagnostic(
@@ -864,9 +864,12 @@ void report_check_restriction(std::map<std::string, ASR::ttype_t*> type_subs,
     }
     if (f->m_return_var) {
         if (!arg->m_return_var) {
-            std::string msg = "The restriction argument " + arg_name
-                + " should have a return value";
-            throw SemanticError(msg, loc);
+            diagnostics.add(diag::Diagnostic(
+                "The restriction argument " + arg_name
+                + " should have a return value",
+                diag::Level::Error, diag::Stage::Semantic, {
+                    diag::Label("", {loc})}));
+            throw SemanticAbort();
         }
         ASR::ttype_t *f_ret = ASRUtils::expr_type(f->m_return_var);
         ASR::ttype_t *arg_ret = ASRUtils::expr_type(arg->m_return_var);
@@ -874,8 +877,8 @@ void report_check_restriction(std::map<std::string, ASR::ttype_t*> type_subs,
             ASR::TypeParameter_t *return_tp
                 = ASR::down_cast<ASR::TypeParameter_t>(f_ret);
             if (!ASRUtils::check_equal_type(type_subs[return_tp->m_param], arg_ret)) {
-                std::string rtype = ASRUtils::type_to_str(type_subs[return_tp->m_param]);
-                std::string atype = ASRUtils::type_to_str(arg_ret);
+                std::string rtype = ASRUtils::type_to_str_fortran(type_subs[return_tp->m_param]);
+                std::string atype = ASRUtils::type_to_str_fortran(arg_ret);
                 diagnostics.add(diag::Diagnostic(
                     "Restriction type mismatch with provided function argument",
                     diag::Level::Error, diag::Stage::Semantic, {
@@ -891,9 +894,12 @@ void report_check_restriction(std::map<std::string, ASR::ttype_t*> type_subs,
         }
     } else {
         if (arg->m_return_var) {
-            std::string msg = "The restriction argument " + arg_name
-                + " should not have a return value";
-            throw SemanticError(msg, loc);
+            diagnostics.add(diag::Diagnostic(
+                "The restriction argument " + arg_name
+                + " should not have a return value",
+                diag::Level::Error, diag::Stage::Semantic, {
+                    diag::Label("", {loc})}));
+            throw SemanticAbort();
         }
     }
     symbol_subs[f_name] = sym_arg;
@@ -962,7 +968,7 @@ public:
             return current_scope->get_symbol(new_sym_name);
         }
 
-        ASR::symbol_t* new_v = ASR::down_cast<ASR::symbol_t>(ASR::make_Variable_t(
+        ASR::symbol_t* new_v = ASR::down_cast<ASR::symbol_t>(ASRUtils::make_Variable_t_util(
             al, x->base.base.loc,
             current_scope, s2c(al, new_sym_name), x->m_dependencies,
             x->n_dependencies, x->m_intent, x->m_symbolic_value,
@@ -1029,7 +1035,7 @@ public:
         ASR::symbol_t *v = current_scope->get_symbol(x->m_name);
         if (!v) {
             ASR::ttype_t *t = substitute_type(x->m_type);
-            v = ASR::down_cast<ASR::symbol_t>(ASR::make_Variable_t(
+            v = ASR::down_cast<ASR::symbol_t>(ASRUtils::make_Variable_t_util(
                 al, x->base.base.loc, current_scope, x->m_name, x->m_dependencies,
                 x->n_dependencies, x->m_intent, x->m_symbolic_value,
                 x->m_value, x->m_storage, t, x->m_type_declaration,
@@ -1204,9 +1210,9 @@ public:
         variable_dependencies_vec.reserve(al, 1);
         ASRUtils::collect_variable_dependencies(al, variable_dependencies_vec, new_type);
 
-        ASR::symbol_t* s = ASR::down_cast<ASR::symbol_t>(ASR::make_Variable_t(al,
+        ASR::symbol_t* s = ASR::down_cast<ASR::symbol_t>(ASRUtils::make_Variable_t_util(al,
             x->base.base.loc, target_scope, s2c(al, x->m_name), variable_dependencies_vec.p,
-            variable_dependencies_vec.size(), x->m_intent, nullptr, nullptr, x->m_storage,
+            variable_dependencies_vec.size(), x->m_intent, x->m_symbolic_value, x->m_value, x->m_storage,
             new_type, nullptr, x->m_abi, x->m_access, x->m_presence, x->m_value_attr));
         target_scope->add_symbol(x->m_name, s);
 
@@ -1259,7 +1265,7 @@ public:
 
         ASR::asr_t* result = ASR::make_Struct_t(al, x->base.base.loc,
             new_scope, s2c(al, new_sym_name), nullptr, 0, data_member_names.p,
-            data_member_names.size(), x->m_abi, x->m_access, x->m_is_packed,
+            data_member_names.size(), nullptr, 0, x->m_abi, x->m_access, x->m_is_packed,
             x->m_is_abstract, nullptr, 0, m_alignment, nullptr);
 
         ASR::symbol_t* s = ASR::down_cast<ASR::symbol_t>(result);
@@ -1365,7 +1371,7 @@ public:
                 std::string struct_name = ASRUtils::symbol_name(s->m_derived_type);
                 if (symbol_subs.find(struct_name) != symbol_subs.end()) {
                     ASR::symbol_t *sym = symbol_subs[struct_name];
-                    return ASRUtils::TYPE(ASR::make_StructType_t(al, ttype->base.loc, sym));
+                    return ASRUtils::TYPE(ASRUtils::make_StructType_t_util(al, ttype->base.loc, sym));
                 }
                 return ttype;
             }
@@ -1750,7 +1756,7 @@ public:
         ASR::expr_t *target = duplicate_expr(x->m_target);
         ASR::expr_t *value = duplicate_expr(x->m_value);
         ASR::stmt_t *overloaded = duplicate_stmt(x->m_overloaded);
-        return ASR::make_Assignment_t(al, x->base.base.loc, target, value, overloaded);
+        return ASRUtils::make_Assignment_t_util(al, x->base.base.loc, target, value, overloaded, false);
     }
 
     /* array_index */
@@ -1785,7 +1791,7 @@ public:
                 std::string struct_name = ASRUtils::symbol_name(s->m_derived_type);
                 if (symbol_subs.find(struct_name) != symbol_subs.end()) {
                     ASR::symbol_t *sym = symbol_subs[struct_name];
-                    ttype = ASRUtils::TYPE(ASR::make_StructType_t(al, s->base.base.loc, sym));
+                    ttype = ASRUtils::TYPE(ASRUtils::make_StructType_t_util(al, s->base.base.loc, sym));
                 }
                 return ttype;
             }
@@ -1885,7 +1891,7 @@ bool check_restriction(std::map<std::string, ASR::ttype_t*> type_subs,
                 std::string rtype = ASRUtils::type_to_str_with_substitution(f_param, type_subs);
                 std::string rvar = ASRUtils::symbol_name(
                                     ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v);
-                std::string atype = ASRUtils::type_to_str(arg_param);
+                std::string atype = ASRUtils::type_to_str_fortran(arg_param);
                 std::string avar = ASRUtils::symbol_name(
                                     ASR::down_cast<ASR::Var_t>(arg->m_args[i])->m_v);
                 diagnostics.add(diag::Diagnostic(
@@ -1919,7 +1925,7 @@ bool check_restriction(std::map<std::string, ASR::ttype_t*> type_subs,
         if (!ASRUtils::types_equal_with_substitution(f_ret, arg_ret, type_subs)) {
             if (report) {
                 std::string rtype = ASRUtils::type_to_str_with_substitution(f_ret, type_subs);
-                std::string atype = ASRUtils::type_to_str(arg_ret);
+                std::string atype = ASRUtils::type_to_str_fortran(arg_ret);
                 diagnostics.add(diag::Diagnostic(
                     "Restriction type mismatch with provided function argument",
                     diag::Level::Error, diag::Stage::Semantic, {
