@@ -64,6 +64,7 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(Isnan)
         INTRINSIC_NAME_CASE(Nearest)
         INTRINSIC_NAME_CASE(CompilerVersion)
+        INTRINSIC_NAME_CASE(CompilerOptions)
         INTRINSIC_NAME_CASE(CommandArgumentCount)
         INTRINSIC_NAME_CASE(Spacing)
         INTRINSIC_NAME_CASE(Modulo)
@@ -117,7 +118,6 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(SubstrIndex)
         INTRINSIC_NAME_CASE(Range)
         INTRINSIC_NAME_CASE(Radix)
-        INTRINSIC_NAME_CASE(IsContiguous)
         INTRINSIC_NAME_CASE(StorageSize)
         INTRINSIC_NAME_CASE(Hypot)
         INTRINSIC_NAME_CASE(SelectedIntKind)
@@ -299,6 +299,8 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Nearest::instantiate_Nearest, &Nearest::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::CompilerVersion),
             {nullptr, &CompilerVersion::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::CompilerOptions),
+            {nullptr, &CompilerOptions::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::CommandArgumentCount),
             {&CommandArgumentCount::instantiate_CommandArgumentCount, &CommandArgumentCount::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Spacing),
@@ -447,8 +449,6 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Sign::instantiate_Sign, &Sign::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Radix),
             {nullptr, &Radix::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::IsContiguous),
-            {nullptr, &IsContiguous::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StorageSize),
             {nullptr, &StorageSize::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Scale),
@@ -642,6 +642,8 @@ namespace IntrinsicElementalFunctionRegistry {
             "nearest"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::CompilerVersion),
             "compiler_version"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::CompilerOptions),
+            "compiler_options"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::CommandArgumentCount),
             "command_argument_count"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Spacing),
@@ -814,8 +816,6 @@ namespace IntrinsicElementalFunctionRegistry {
             "ishftc"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Radix),
             "radix"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::IsContiguous),
-            "is_contiguous"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StorageSize),
             "storage_size"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Scale),
@@ -974,7 +974,8 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"trailz", {&Trailz::create_Trailz, &Trailz::eval_Trailz}},
                 {"isnan", {&Isnan::create_Isnan, &Isnan::eval_Isnan}},
                 {"nearest", {&Nearest::create_Nearest, &Nearest::eval_Nearest}},
-                {"compiler_version", {&CompilerVersion::create_CompilerVersion, &CompilerVersion::eval_CompilerVersion}},
+                {"_lfortran_compiler_version", {&CompilerVersion::create_CompilerVersion, &CompilerVersion::eval_CompilerVersion}},
+                {"compiler_options", {&CompilerOptions::create_CompilerOptions, &CompilerOptions::eval_CompilerOptions}},
                 {"command_argument_count", {&CommandArgumentCount::create_CommandArgumentCount, nullptr}},
                 {"spacing", {&Spacing::create_Spacing, &Spacing::eval_Spacing}},
                 {"modulo", {&Modulo::create_Modulo, &Modulo::eval_Modulo}},
@@ -1052,7 +1053,6 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"min", {&Min::create_Min, &Min::eval_Min}},
                 {"ishftc", {&Ishftc::create_Ishftc, &Ishftc::eval_Ishftc}},
                 {"radix", {&Radix::create_Radix, &Radix::eval_Radix}},
-                {"is_contiguous", {&IsContiguous::create_IsContiguous, &IsContiguous::eval_IsContiguous}},
                 {"out_of_range", {&OutOfRange::create_OutOfRange, &OutOfRange::eval_OutOfRange}},
                 {"storage_size", {&StorageSize::create_StorageSize, &StorageSize::eval_StorageSize}},
                 {"scale", {&Scale::create_Scale, &Scale::eval_Scale}},
