@@ -113,7 +113,15 @@ namespace RandomSeed {
     }
 
     static inline ASR::asr_t* create_RandomSeed(Allocator& al, const Location& loc, Vec<ASR::expr_t*>& args, diag::Diagnostics& /*diag*/) {
-        Vec<ASR::expr_t*> m_args; m_args.reserve(al, 1); m_args.push_back(al, args[0]);
+        Vec<ASR::expr_t*> m_args; m_args.reserve(al, 0);
+        ASRBuilder b(al, loc);
+        for (int i = 0; i < int(args.size()); i++) {
+            if(args[i]) {
+                m_args.push_back(al, args[i]);
+            } else {
+                m_args.push_back(al, b.f32(1));
+            }
+        }
         return ASR::make_IntrinsicImpureSubroutine_t(al, loc, static_cast<int64_t>(IntrinsicImpureSubroutines::RandomSeed), m_args.p, m_args.n, 0);
     }
 
@@ -121,31 +129,69 @@ namespace RandomSeed {
             SymbolTable *scope, Vec<ASR::ttype_t*>& arg_types,
             Vec<ASR::call_arg_t>& new_args, int64_t /*overload_id*/) {
 
-        std::string c_func_name = "_lfortran_random_seed";
+        std::string c_func_name_1 = "_lfortran_random_seed";
+        std::string c_func_name_2 = "_lfortran_dp_rand_num";
         std::string new_name = "_lcompilers_random_seed_";
         declare_basic_variables(new_name);
-        fill_func_arg_sub("r", arg_types[0], InOut);
-        SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
-        Vec<ASR::expr_t*> args_1; args_1.reserve(al, 1);
-        ASR::expr_t *arg = b.Variable(fn_symtab_1, "n", arg_types[0],
-            ASR::intentType::In, ASR::abiType::BindC, true);
-        args_1.push_back(al, arg);
-
-        ASR::expr_t *return_var_1 = b.Variable(fn_symtab_1, c_func_name,
-           ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(arg_types[0])),
-           ASRUtils::intent_return_var, ASR::abiType::BindC, false);
-           
-        SetChar dep_1; dep_1.reserve(al, 1);
-        Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
-        ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
-            body_1, return_var_1, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
-        fn_symtab->add_symbol(c_func_name, s);
-        dep.push_back(al, s2c(al, c_func_name));
-
-        Vec<ASR::expr_t*> call_args; call_args.reserve(al, 1);
-        call_args.push_back(al, b.i32(8));
-        body.push_back(al, b.Assignment(args[0], b.Call(s, call_args, arg_types[0])));
-        
+        int flag = 0;
+        if (!is_real(*arg_types[0])) {
+            fill_func_arg_sub("size", arg_types[0], InOut);
+            ASR::symbol_t *s_1 = b.create_c_func_subroutines(c_func_name_1, fn_symtab, 1, arg_types[0]);
+            fn_symtab->add_symbol(c_func_name_1, s_1);
+            dep.push_back(al, s2c(al, c_func_name_1));
+            Vec<ASR::expr_t*> call_args; call_args.reserve(al, 1);
+            call_args.push_back(al, b.i32(8));
+            body.push_back(al, b.Assignment(args[0], b.Call(s_1, call_args, arg_types[0])));
+        } else {
+            fill_func_arg_sub("size", real32, InOut);
+            body.push_back(al, b.Assignment(args[0], b.f32(0)));
+        }
+        if (!is_real(*arg_types[1])) {
+            flag = 1;
+            fill_func_arg_sub("put", arg_types[1], InOut);
+            body.push_back(al, b.Assignment(args[1], args[1]));
+        } else {
+            fill_func_arg_sub("put", real32, InOut);
+            body.push_back(al, b.Assignment(args[1], b.f32(0)));
+        }
+        if (!is_real(*arg_types[2])) {
+            fill_func_arg_sub("get", arg_types[2], InOut);
+            if (flag == 1) {
+                body.push_back(al, b.Assignment(args[2], args[1]));
+            } else {
+                std::vector<LCompilers::ASR::expr_t *> vals;
+                std::string c_func = c_func_name_2;
+                int kind = ASRUtils::extract_kind_from_ttype_t(arg_types[2]);
+                if ( is_real(*arg_types[2]) ) {
+                    if (kind == 4) {
+                        c_func = "_lfortran_sp_rand_num";
+                    } else {
+                        c_func = "_lfortran_dp_rand_num";
+                    }
+                } else if ( is_integer(*arg_types[2]) ) {
+                    if (kind == 4) {
+                        c_func = "_lfortran_int32_rand_num";
+                    } else {
+                        c_func = "_lfortran_int64_rand_num";
+                    }
+                }
+                ASR::symbol_t *s_2 = b.create_c_func_subroutines(c_func, fn_symtab, 0, arg_types[2]);
+                fn_symtab->add_symbol(c_func, s_2);
+                dep.push_back(al, s2c(al, c_func));
+                Vec<ASR::expr_t*> call_args2; call_args2.reserve(al, 0);
+                ASR::ttype_t* elem_type = extract_type(arg_types[2]);
+                for (int i = 0; i < 8; i++) {
+                    auto xx = declare("i_" + std::to_string(i), elem_type, Local);
+                    body.push_back(al, b.Assignment(xx, b.Call(s_2, call_args2, int32)));
+                    vals.push_back(xx);
+                }
+                body.push_back(al, b.Assignment(args[2], b.ArrayConstant(vals, extract_type(arg_types[2]), false)));
+            }
+        } else {
+            fill_func_arg_sub("get", real32, InOut);
+            body.push_back(al, b.Assignment(args[2], b.f32(0)));
+        }
+       
         ASR::symbol_t *new_symbol = make_ASR_Function_t(fn_name, fn_symtab, dep, args,
             body, nullptr, ASR::abiType::Source, ASR::deftypeType::Implementation, nullptr);
         scope->add_symbol(fn_name, new_symbol);
@@ -236,7 +282,7 @@ namespace RandomNumber {
         std::string new_name = "_lcompilers_random_number_";
 
         declare_basic_variables(new_name);
-        fill_func_arg_sub("r", arg_types[0], InOut);
+        fill_func_arg_sub("r", ASRUtils::duplicate_type_with_empty_dims(al, arg_types[0]), InOut);
         SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
         Vec<ASR::expr_t*> args_1; args_1.reserve(al, 0);
         ASR::expr_t *return_var_1 = b.Variable(fn_symtab_1, c_func_name,
@@ -460,17 +506,27 @@ namespace SystemClock {
     }
 
     static inline ASR::asr_t* create_SystemClock(Allocator& al, const Location& loc, Vec<ASR::expr_t*>& args, diag::Diagnostics& /*diag*/) {
+        int64_t count_id = 0, count_rate_id = 1, count_max_id = 2, count_count_rate_id = 3, count_count_max_id = 4, count_rate_count_max_id = 5, count_count_rate_count_max_id = 6;
+        int64_t overload_id = -1;
         Vec<ASR::expr_t*> m_args; m_args.reserve(al, args.size());
-        m_args.push_back(al, args[0]);
-        for (int i = 1; i < int(args.size()); i++) {
+        ASRBuilder b(al, loc);
+        if(args[0]) overload_id = count_id;
+        if(args[1]) overload_id = count_rate_id;
+        if(args[2]) overload_id = count_max_id;
+        if(args[0] && args[1]) overload_id = count_count_rate_id;
+        if(args[0] && args[2]) overload_id = count_count_max_id;
+        if(args[1] && args[2]) overload_id = count_rate_count_max_id;
+        if(args[0] && args[1] && args[2]) overload_id = count_count_rate_count_max_id;
+        for (int i = 0; i < int(args.size()); i++) {
             if(args[i]) m_args.push_back(al, args[i]);
+            else m_args.push_back(al, b.i32(1));
         }
-        return ASR::make_IntrinsicImpureSubroutine_t(al, loc, static_cast<int64_t>(IntrinsicImpureSubroutines::SystemClock), m_args.p, m_args.n, 0);
+        return ASR::make_IntrinsicImpureSubroutine_t(al, loc, static_cast<int64_t>(IntrinsicImpureSubroutines::SystemClock), m_args.p, m_args.n, overload_id);
     }
 
     static inline ASR::stmt_t* instantiate_SystemClock(Allocator &al, const Location &loc,
             SymbolTable *scope, Vec<ASR::ttype_t*>& arg_types,
-            Vec<ASR::call_arg_t>& new_args, int64_t /*overload_id*/) {
+            Vec<ASR::call_arg_t>& new_args, int64_t overload_id) {
 
         std::string c_func_name_1 = "_lfortran_i32sys_clock_count";
         std::string c_func_name_2 = "_lfortran_i32sys_clock_count_rate";
@@ -478,8 +534,7 @@ namespace SystemClock {
         std::string new_name = "_lcompilers_system_clock_";
         declare_basic_variables(new_name);
         Vec<ASR::expr_t*> call_args; call_args.reserve(al, 0);
-
-        if (arg_types.size() > 0) {
+        if (overload_id == 0 || overload_id == 3 || overload_id == 4 || overload_id == 6) {
             if (ASRUtils::extract_kind_from_ttype_t(arg_types[0]) == 8) {
                 c_func_name_1 = "_lfortran_i64sys_clock_count";
             }
@@ -488,9 +543,11 @@ namespace SystemClock {
             fn_symtab->add_symbol(c_func_name_1, s_1);
             dep.push_back(al, s2c(al, c_func_name_1));
             body.push_back(al, b.Assignment(args[0], b.Call(s_1, call_args, arg_types[0])));
-
+        } else {
+            fill_func_arg_sub("count", int32, InOut);
+            body.push_back(al, b.Assignment(args[0], b.i32(0)));
         }
-        if (arg_types.size() > 1) {
+        if (overload_id == 1 || overload_id == 3 || overload_id == 5 || overload_id == 6) {
             if (ASRUtils::extract_kind_from_ttype_t(arg_types[1]) == 8) {
                 if (is_real(*arg_types[1])) {
                     c_func_name_2 = "_lfortran_i64r64sys_clock_count_rate";
@@ -505,8 +562,11 @@ namespace SystemClock {
             fn_symtab->add_symbol(c_func_name_2, s_2);
             dep.push_back(al, s2c(al, c_func_name_2));
             body.push_back(al, b.Assignment(args[1], b.Call(s_2, call_args, arg_types[1])));
+        } else {
+            fill_func_arg_sub("count_rate", int32, InOut);
+            body.push_back(al, b.Assignment(args[1], b.i32(0)));
         }
-        if (arg_types.size() > 2) {
+        if (overload_id == 2 || overload_id == 4 || overload_id == 5 || overload_id == 6) {
             if (ASRUtils::extract_kind_from_ttype_t(arg_types[2]) == 8) {
                 c_func_name_3 = "_lfortran_i64sys_clock_count_max";
             }
@@ -515,6 +575,9 @@ namespace SystemClock {
             fn_symtab->add_symbol(c_func_name_3, s_3);
             dep.push_back(al, s2c(al, c_func_name_3));
             body.push_back(al, b.Assignment(args[2], b.Call(s_3, call_args, arg_types[2])));
+        } else {
+            fill_func_arg_sub("count_max", int32, InOut);
+            body.push_back(al, b.Assignment(args[2], b.i32(0)));
         }
 
         ASR::symbol_t *new_symbol = make_ASR_Function_t(fn_name, fn_symtab, dep, args,
@@ -547,9 +610,13 @@ namespace DateAndTime {
 
     static inline ASR::asr_t* create_DateAndTime(Allocator& al, const Location& loc, Vec<ASR::expr_t*>& args, diag::Diagnostics& /*diag*/) {
         Vec<ASR::expr_t*> m_args; m_args.reserve(al, args.size());
-        m_args.push_back(al, args[0]);
-        for (int i = 1; i < int(args.size()); i++) {
-            if(args[i]) m_args.push_back(al, args[i]);
+        ASRBuilder b(al, loc);
+        for (int i = 0; i < int(args.size()); i++) {
+            if(args[i]) {
+                m_args.push_back(al, args[i]);
+            } else {
+                m_args.push_back(al, b.f32(1));
+            }
         }
         return ASR::make_IntrinsicImpureSubroutine_t(al, loc, static_cast<int64_t>(IntrinsicImpureSubroutines::DateAndTime), m_args.p, m_args.n, 0);
     }
@@ -566,33 +633,53 @@ namespace DateAndTime {
         declare_basic_variables(new_name);
         Vec<ASR::expr_t*> call_args; call_args.reserve(al, 0);
 
-        if (arg_types.size() > 0) {
+        if (!is_real(*arg_types[0])) {
             fill_func_arg_sub("date", arg_types[0], InOut);
             ASR::symbol_t *s_1 = b.create_c_func_subroutines(c_func_name_1, fn_symtab, 0, arg_types[0]);
             fn_symtab->add_symbol(c_func_name_1, s_1);
             dep.push_back(al, s2c(al, c_func_name_1));
             body.push_back(al, b.Assignment(args[0], b.Call(s_1, call_args, arg_types[0])));
+        } else {
+            fill_func_arg_sub("date", real32, InOut);
+            body.push_back(al, b.Assignment(args[0], b.f32(0)));
         }
-        if (arg_types.size() > 1) {
+        if (!is_real(*arg_types[1])) {
             fill_func_arg_sub("time", arg_types[1], InOut);
             ASR::symbol_t *s_2 = b.create_c_func_subroutines(c_func_name_2, fn_symtab, 0, arg_types[1]);
             fn_symtab->add_symbol(c_func_name_2, s_2);
             dep.push_back(al, s2c(al, c_func_name_2));
             body.push_back(al, b.Assignment(args[1], b.Call(s_2, call_args, arg_types[1])));
+        }  else {
+            fill_func_arg_sub("time", real32, InOut);
+            body.push_back(al, b.Assignment(args[1], b.f32(0)));
         }
-        if (arg_types.size() > 2) {
+        if (!is_real(*arg_types[2])) {
             fill_func_arg_sub("zone", arg_types[2], InOut);
             ASR::symbol_t *s_3 = b.create_c_func_subroutines(c_func_name_3, fn_symtab, 0, arg_types[2]);
             fn_symtab->add_symbol(c_func_name_3, s_3);
             dep.push_back(al, s2c(al, c_func_name_3));
             body.push_back(al, b.Assignment(args[2], b.Call(s_3, call_args, arg_types[2])));
+        } else {
+            fill_func_arg_sub("zone", real32, InOut);
+            body.push_back(al, b.Assignment(args[2], b.f32(0)));
         }
-        if (arg_types.size() > 3) {
+        if (!is_real(*arg_types[3])) {
             fill_func_arg_sub("values", arg_types[3], InOut);
-            ASR::symbol_t *s_4 = b.create_c_func_subroutines(c_func_name_4, fn_symtab, 0, arg_types[3]);
+            std::vector<LCompilers::ASR::expr_t *> vals;
+            ASR::symbol_t *s_4 = b.create_c_func_subroutines(c_func_name_4, fn_symtab, 1, int32);
             fn_symtab->add_symbol(c_func_name_4, s_4);
             dep.push_back(al, s2c(al, c_func_name_4));
-            // body.push_back(al, b.Assignment(args[3], b.Call(s_4, call_args, arg_types[3])));
+            for (int i = 0; i < 8; i++) {
+                Vec<ASR::expr_t*> call_args2; call_args2.reserve(al, 1);
+                call_args2.push_back(al, b.i32(i+1));
+                auto xx = declare("i_" + std::to_string(i), int32, Local);
+                body.push_back(al, b.Assignment(xx, b.Call(s_4, call_args2, int32)));
+                vals.push_back(xx);
+            }
+            body.push_back(al, b.Assignment(args[3], b.ArrayConstant(vals, extract_type(arg_types[3]), false)));
+        } else {
+            fill_func_arg_sub("values", real32, InOut);
+            body.push_back(al, b.Assignment(args[3], b.f32(0)));
         }
         ASR::symbol_t *new_symbol = make_ASR_Function_t(fn_name, fn_symtab, dep, args,
             body, nullptr, ASR::abiType::Source, ASR::deftypeType::Implementation, nullptr);
@@ -688,45 +775,51 @@ namespace ExecuteCommandLine {
     static inline ASR::stmt_t* instantiate_ExecuteCommandLine(Allocator &al, const Location &loc,
             SymbolTable *scope, Vec<ASR::ttype_t*>& arg_types,
             Vec<ASR::call_arg_t>& new_args, int64_t /*overload_id*/) {
-                
+
         std::string c_func_name = "_lfortran_exec_command";
         std::string new_name = "_lcompilers_execute_command_line_";
-        declare_basic_variables(new_name);
-        fill_func_arg_sub("command", arg_types[0], InOut);
-        if (arg_types.size() == 2) {
-            fill_func_arg_sub("wait", arg_types[1], InOut);
-        } else if (arg_types.size() == 3) {
-            fill_func_arg_sub("exitstat", arg_types[2], InOut);
-        } else if (arg_types.size() == 4) {
-            fill_func_arg_sub("cmdstat", arg_types[3], InOut);
-        } else if (arg_types.size() == 5) {
-            fill_func_arg_sub("cmdmsg", arg_types[4], InOut);
+        ASR::symbol_t* s = scope->get_symbol(new_name);
+        if (s) {
+            ASRBuilder b(al, loc);
+            return b.SubroutineCall(s, new_args);
+        } else {
+            declare_basic_variables(new_name);
+            fill_func_arg_sub("command", arg_types[0], InOut);
+            if (arg_types.size() == 2) {
+                fill_func_arg_sub("wait", arg_types[1], InOut);
+            } else if (arg_types.size() == 3) {
+                fill_func_arg_sub("exitstat", arg_types[2], InOut);
+            } else if (arg_types.size() == 4) {
+                fill_func_arg_sub("cmdstat", arg_types[3], InOut);
+            } else if (arg_types.size() == 5) {
+                fill_func_arg_sub("cmdmsg", arg_types[4], InOut);
+            }
+
+            SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
+            Vec<ASR::expr_t*> args_1; args_1.reserve(al, 1);
+            ASR::expr_t *arg = b.Variable(fn_symtab_1, "n", arg_types[0],
+                ASR::intentType::InOut, ASR::abiType::BindC, true);
+            args_1.push_back(al, arg);
+
+            ASR::expr_t *return_var_1 = b.Variable(fn_symtab_1, c_func_name,
+            ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(arg_types[0])),
+            ASRUtils::intent_return_var, ASR::abiType::BindC, false);
+
+            SetChar dep_1; dep_1.reserve(al, 1);
+            Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
+            ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
+                body_1, return_var_1, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
+            fn_symtab->add_symbol(c_func_name, s);
+            dep.push_back(al, s2c(al, c_func_name));
+
+            Vec<ASR::expr_t*> call_args; call_args.reserve(al, 1);
+            call_args.push_back(al, args[0]);
+            body.push_back(al, b.Assignment(args[0], b.Call(s, call_args, arg_types[0])));
+            ASR::symbol_t *new_symbol = make_ASR_Function_t(fn_name, fn_symtab, dep, args,
+                body, nullptr, ASR::abiType::Intrinsic, ASR::deftypeType::Implementation, nullptr);
+            scope->add_symbol(fn_name, new_symbol);
+            return b.SubroutineCall(new_symbol, new_args);
         }
-
-        SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
-        Vec<ASR::expr_t*> args_1; args_1.reserve(al, 1);
-        ASR::expr_t *arg = b.Variable(fn_symtab_1, "n", arg_types[0],
-            ASR::intentType::InOut, ASR::abiType::BindC, true);
-        args_1.push_back(al, arg);
-
-        ASR::expr_t *return_var_1 = b.Variable(fn_symtab_1, c_func_name,
-           ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(arg_types[0])),
-           ASRUtils::intent_return_var, ASR::abiType::BindC, false);
-           
-        SetChar dep_1; dep_1.reserve(al, 1);
-        Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
-        ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
-            body_1, return_var_1, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
-        fn_symtab->add_symbol(c_func_name, s);
-        dep.push_back(al, s2c(al, c_func_name));
-
-        Vec<ASR::expr_t*> call_args; call_args.reserve(al, 1);
-        call_args.push_back(al, args[0]);
-        body.push_back(al, b.Assignment(args[0], b.Call(s, call_args, arg_types[0])));
-        ASR::symbol_t *new_symbol = make_ASR_Function_t(fn_name, fn_symtab, dep, args,
-            body, nullptr, ASR::abiType::Source, ASR::deftypeType::Implementation, nullptr);
-        scope->add_symbol(fn_name, new_symbol);
-        return b.SubroutineCall(new_symbol, new_args);
     }
 
 } // namespace ExecuteCommandLine
