@@ -182,7 +182,7 @@ class FunctionSubroutineCallVisitor: public ASR::BaseWalkVisitor<FunctionSubrout
         std::vector<int> &array_variable_indices;
         std::vector<std::string> &array_variables;
         std::map<int, std::map<std::string, std::vector<ASR::symbol_t*>>> &scoped_array_variable_map;
-    
+
     public:
         FunctionSubroutineCallVisitor(std::string function_name_, std::vector<SymbolTable*> &scopes_,
             std::vector<int> &array_variable_indices_,
@@ -191,7 +191,7 @@ class FunctionSubroutineCallVisitor: public ASR::BaseWalkVisitor<FunctionSubrout
             function_name(function_name_), scopes(scopes_), array_variable_indices(array_variable_indices_),
             array_variables(array_variables_),
             scoped_array_variable_map(scoped_array_variable_map_) {}
-        
+
         void visit_Program(const ASR::Program_t &x) {
             ASR::Program_t& xx = const_cast<ASR::Program_t&>(x);
             SymbolTable* current_scope_copy = current_scope;
@@ -346,26 +346,91 @@ class DoConcurrentStatementVisitor : public ASR::CallReplacerOnExpressionsVisito
         replacer.replace_expr(*current_expr);
     }
 
-    void visit_FunctionCall(const ASR::FunctionCall_t &x) {
-        ASR::FunctionCall_t* x_copy = const_cast<ASR::FunctionCall_t*>(&x);
+    template <typename T>
+    void visit_Call(const T &x) {
+        T* x_copy = const_cast<T*>(&x);
+        ASR::Function_t* fn = ASR::down_cast<ASR::Function_t>(
+                            ASRUtils::symbol_get_past_external(x_copy->m_name));
+        ASR::asr_t* asr_owner = ASRUtils::symbol_parent_symtab(x.m_name)->asr_owner;
+        ASR::symbol_t* fun_sym_for_module = nullptr;
+        char* module_name = nullptr;
+        // Steps:
+        // Create a module add it to current_scope->parent symtab
+        // Add func to that module symtab
+        // Overwrite External symbol to x's asr_owner's symtab
+        if (ASR::is_a<ASR::Program_t>(*ASR::down_cast<ASR::symbol_t>(asr_owner))) {
+            ASRUtils::SymbolDuplicator duplicator(al);
+            SymbolTable* module_scope = al.make_new<SymbolTable>(current_scope->parent);
+
+            module_name = s2c(al, current_scope->parent->get_unique_name("lcompilers_user_defined_functions"));
+            ASR::asr_t* mo = ASR::make_Module_t(
+                                al, x.base.base.loc, module_scope,
+                                s2c(al, module_name), nullptr,
+                                0, false, false);
+            if (current_scope->parent->get_symbol(module_name) == nullptr) {
+                current_scope->parent->add_symbol(module_name, ASR::down_cast<ASR::symbol_t>(mo));
+            }
+
+            ASR::Module_t* module = ASR::down_cast<ASR::Module_t>(ASR::down_cast<ASR::symbol_t>(mo));
+            fun_sym_for_module = duplicator.duplicate_Function(fn, module_scope);
+            module->m_symtab->add_symbol(fn->m_name, fun_sym_for_module);
+
+            ASR::asr_t* ext_fn = ASR::make_ExternalSymbol_t(
+                                al,
+                                x.base.base.loc,
+                                ASRUtils::symbol_parent_symtab(x.m_name),
+                                fn->m_name,
+                                fun_sym_for_module,
+                                s2c(al, module_name),
+                                nullptr,
+                                0,
+                                x_copy->m_original_name
+                                    ? ASRUtils::symbol_name(x_copy->m_original_name)
+                                    : ASRUtils::symbol_name(x_copy->m_name),
+                                ASR::accessType::Public);
+            ASR::Program_t* program = ASR::down_cast<ASR::Program_t>(
+                                    ASR::down_cast<ASR::symbol_t>(asr_owner));
+            program->m_symtab->add_or_overwrite_symbol(fn->m_name,
+                                                       ASR::down_cast<ASR::symbol_t>(ext_fn));
+        }
+
         ASR::symbol_t* func_sym = current_scope->get_symbol(ASRUtils::symbol_name(x.m_name));
         if (func_sym == nullptr) {
-            // this means we have a user defined function and need to create an interface for it
-            ASR::Function_t* func = ASR::down_cast<ASR::Function_t>(x.m_name);
-            ASRUtils::SymbolDuplicator duplicator(al);
-            // This will create trouble when we have something like x(n) inside function
-            // duplicator does not duplicate type of variables inside function
-            func_sym = duplicator.duplicate_Function(func, current_scope);
-            current_scope->add_symbol(ASRUtils::symbol_name(func_sym), func_sym);
-            ASR::Function_t* new_func = ASR::down_cast<ASR::Function_t>(func_sym);
-            new_func->m_body = nullptr; new_func->n_body = 0;
-            ASR::down_cast<ASR::FunctionType_t>(new_func->m_function_signature)->m_deftype = ASR::deftypeType::Interface;
+            if (ASR::is_a<ASR::Program_t>(*ASR::down_cast<ASR::symbol_t>(asr_owner))) {
+                ASR::asr_t* ext_fn = ASR::make_ExternalSymbol_t(
+                                        al,
+                                        x.base.base.loc,
+                                        current_scope,
+                                        fn->m_name,
+                                        fun_sym_for_module,
+                                        s2c(al, module_name),
+                                        nullptr,
+                                        0,
+                                        x_copy->m_original_name
+                                            ? ASRUtils::symbol_name(x_copy->m_original_name)
+                                            : ASRUtils::symbol_name(x_copy->m_name),
+                                        ASR::accessType::Public);
+                current_scope->add_or_overwrite_symbol(fn->m_name,
+                                                       ASR::down_cast<ASR::symbol_t>(ext_fn));
+                func_sym = current_scope->get_symbol(fn->m_name);
+            } else if (ASR::is_a<ASR::Module_t>(*ASR::down_cast<ASR::symbol_t>(asr_owner))) {
+                func_sym = current_scope->resolve_symbol(fn->m_name);
+            }
         }
         LCOMPILERS_ASSERT(func_sym != nullptr);
         x_copy->m_name = func_sym;
         x_copy->m_original_name = func_sym;
+    }
+
+    void visit_FunctionCall(const ASR::FunctionCall_t &x) {
+        visit_Call(x);
         CallReplacerOnExpressionsVisitor::visit_FunctionCall(x);
     }
+
+    void visit_SubroutineCall(const ASR::SubroutineCall_t &x) {
+        visit_Call(x);
+        CallReplacerOnExpressionsVisitor::visit_SubroutineCall(x);
+    }
 };
 
 class InvolvedSymbolsCollector:
@@ -395,23 +460,58 @@ public:
     void replace_Var(ASR::Var_t *x) {
         x->m_v = fn_scope.get_symbol(ASRUtils::symbol_name(x->m_v));
     }
+
+    void replace_FunctionCall(ASR::FunctionCall_t* x) {
+        x->m_name = fn_scope.get_symbol(ASRUtils::symbol_name(x->m_name));
+        if (x->m_original_name) x->m_original_name = fn_scope.get_symbol(ASRUtils::symbol_name(x->m_original_name));
+    }
 };
 
 // Expression visitor to call the replacer
-class DoConcurrentBodyVisitor:
-    public ASR::CallReplacerOnExpressionsVisitor<DoConcurrentBodyVisitor> {
+class ReplaceSymbolsVisitor:
+    public ASR::CallReplacerOnExpressionsVisitor<ReplaceSymbolsVisitor> {
 
 private:
     ReplaceSymbols replacer;
 
 public:
-    DoConcurrentBodyVisitor(SymbolTable &fn_scope): replacer(fn_scope) { }
+    ReplaceSymbolsVisitor(SymbolTable &fn_scope): replacer(fn_scope) { }
 
     void call_replacer() {
         replacer.current_expr = current_expr;
         replacer.replace_expr(*current_expr);
     }
 
+    void visit_FunctionCall(const ASR::FunctionCall_t &x) {
+        replacer.replace_expr(&const_cast<ASR::FunctionCall_t*>(&x)->base);
+    }
+};
+
+class ReplaceStatements: public ASR::BaseStmtReplacer<ReplaceStatements> {
+private:
+    SymbolTable &scope;
+
+public:
+    ReplaceStatements(SymbolTable &scope) : scope(scope) {}
+
+    void replace_SubroutineCall(ASR::SubroutineCall_t* x) {
+        x->m_name = scope.get_symbol(ASRUtils::symbol_name(x->m_name));
+        if (x->m_original_name) x->m_original_name = scope.get_symbol(ASRUtils::symbol_name(x->m_original_name));
+    }
+
+};
+
+class ReplaceStatementsVisitor: public ASR::CallReplacerOnExpressionsVisitor<ReplaceStatements> {
+private:
+    ReplaceStatements replacer;
+
+public:
+    ReplaceStatementsVisitor(SymbolTable &scope) : replacer(scope) {}
+
+    void visit_SubroutineCall(const ASR::SubroutineCall_t &x) {
+        replacer.replace_stmt(&const_cast<ASR::SubroutineCall_t*>(&x)->base);
+    }
+
 };
 
 class DoConcurrentVisitor :
@@ -698,7 +798,7 @@ class DoConcurrentVisitor :
             std::string suffix = thread_data_module_name.substr(18);
             std::string thread_data_name = "thread_data" + suffix;
             ASR::symbol_t* thread_data_struct = ASR::down_cast<ASR::symbol_t>(ASR::make_Struct_t(al, loc,
-                current_scope, s2c(al, thread_data_name), nullptr, 0, involved_symbols_set.p, involved_symbols_set.n, ASR::abiType::Source,
+                current_scope, s2c(al, thread_data_name), nullptr, 0, involved_symbols_set.p, involved_symbols_set.n, nullptr, 0, ASR::abiType::Source,
                 ASR::accessType::Public, false, false, nullptr, 0, nullptr, nullptr));
             current_scope->parent->add_symbol(thread_data_name, thread_data_struct);
             current_scope = parent_scope;
@@ -750,7 +850,7 @@ class DoConcurrentVisitor :
             LCOMPILERS_ASSERT(data_expr != nullptr);
 
             // create tdata variable: `type(thread_data), pointer :: tdata`
-            ASR::expr_t* tdata_expr = b.Variable(current_scope, "tdata", ASRUtils::TYPE(ASR::make_Pointer_t(al, loc, ASRUtils::TYPE(ASR::make_StructType_t(al, loc, thread_data_sym)))),
+            ASR::expr_t* tdata_expr = b.Variable(current_scope, "tdata", ASRUtils::TYPE(ASR::make_Pointer_t(al, loc, ASRUtils::TYPE(ASRUtils::make_StructType_t_util(al, loc, thread_data_sym)))),
                     ASR::intentType::Local, ASR::abiType::BindC);
             LCOMPILERS_ASSERT(tdata_expr != nullptr);
 
@@ -869,7 +969,7 @@ class DoConcurrentVisitor :
                 dimension_lengths.push_back(length);
                 total_iterations = b.Mul(total_iterations, length);
             }
-  
+
             // always this shall be IntegerBinOp_t
             ASR::expr_t* loop_length = total_iterations;
             // ASR::expr_t* loop_length = b.Add(b.Sub(loop_head.m_end, loop_head.m_start), b.i32(1));
@@ -968,7 +1068,7 @@ class DoConcurrentVisitor :
                     loc,2,mod_args.p, 2, 0, ASRUtils::expr_type(dimension_lengths[i]), nullptr)),head.m_start);
                 } else {
                     // Intermediate loop variable -> iy = ((I / ((nz - az 1) * (nk - ak 1))) % (ny - ay +1)) ay
-                    ASR::expr_t* product_of_next_dimensions = b.i32(1); 
+                    ASR::expr_t* product_of_next_dimensions = b.i32(1);
                     for (size_t j = i + 1 ; j <do_loop.n_head; ++j) {
                         product_of_next_dimensions = b.Mul(product_of_next_dimensions, dimension_lengths[j]);
                     }
@@ -1059,7 +1159,7 @@ class DoConcurrentVisitor :
                 ASR::deftypeType::Implementation, nullptr, false, false, false, false, false,
                 nullptr, 0,
                 false, false, false, nullptr));
-            
+
             current_scope->parent->add_symbol(ASRUtils::symbol_name(function), function);
             current_scope = current_scope_copy;
             return function;
@@ -1335,14 +1435,14 @@ class DoConcurrentVisitor :
                         loc, fn_scope, s2c(al, sym_name), nullptr, 0,
                         ASR::intentType::InOut, nullptr, nullptr,
                         ASR::storage_typeType::Default,
-                        ASRUtils::duplicate_type_with_empty_dims(al, sym_type),
+                        ASRUtils::duplicate_type(al, sym_type),
                         nullptr, ASR::abiType::Source, ASR::accessType::Private,
                         ASR::presenceType::Required, false));
                     fn_scope->add_symbol(sym_name, sym);
                     fn_args.push_back(al, ASRUtils::EXPR(ASR::make_Var_t(al, loc, sym)));
                 }
 
-                DoConcurrentBodyVisitor v(*fn_scope);
+                ReplaceSymbolsVisitor v(*fn_scope);
                 v.visit_DoConcurrentLoop(x);
 
                 Vec<ASR::stmt_t *> fn_body; fn_body.reserve(al, 1);
@@ -1353,7 +1453,7 @@ class DoConcurrentVisitor :
                 ASR::symbol_t* function = ASR::down_cast<ASR::symbol_t>(
                     ASRUtils::make_Function_t_util(al, loc, fn_scope,
                     s2c(al, fn_name), nullptr, 0, fn_args.p, fn_args.n,
-                    fn_body.p, fn_body.n, nullptr, ASR::abiType::Source,
+                    fn_body.p, fn_body.n, nullptr, ASR::abiType::BindC,
                     ASR::accessType::Public, ASR::deftypeType::Implementation,
                     nullptr, false, false, false, false, false, nullptr, 0,
                     false, false, false, nullptr));
@@ -1361,13 +1461,33 @@ class DoConcurrentVisitor :
 
                 current_scope = scope_copy;
 
-                ASR::symbol_t *es = ASR::down_cast<ASR::symbol_t>(
-                    ASR::make_ExternalSymbol_t(al, loc, current_scope,
-                    s2c(al, fn_name), function, s2c(al, mod_name),
-                    nullptr, 0, s2c(al, fn_name), ASR::accessType::Public));
-                current_scope->add_symbol(fn_name, es);
-                pass_result.push_back(al, ASRUtils::STMT(ASR::make_SubroutineCall_t(
-                    al, loc, es, es, call_args.p, call_args.n, nullptr)));
+                SymbolTable *fnI_scope{al.make_new<SymbolTable>(current_scope)};
+                Vec<ASR::expr_t *> fnI_args;
+                fnI_args.reserve(al, involved_symbols.size());
+                for (auto &[sym_name, sym_type]: involved_symbols) {
+                    ASR::symbol_t *sym{ASR::down_cast<ASR::symbol_t>(
+                        ASRUtils::make_Variable_t_util(al, loc, fnI_scope,
+                        s2c(al, sym_name), nullptr, 0, ASR::intentType::InOut,
+                        nullptr, nullptr, ASR::storage_typeType::Default,
+                        ASRUtils::duplicate_type(al, sym_type),
+                        nullptr, ASR::abiType::Source, ASR::accessType::Private,
+                        ASR::presenceType::Required, false))};
+                    fnI_scope->add_symbol(sym_name, sym);
+                    fnI_args.push_back(al, ASRUtils::EXPR(
+                        ASR::make_Var_t(al, loc, sym)));
+                }
+
+                ASR::symbol_t* fnInterface = ASR::down_cast<ASR::symbol_t>(
+                    ASRUtils::make_Function_t_util(al, loc, fnI_scope,
+                    s2c(al, fn_name), nullptr, 0, fnI_args.p, fnI_args.n,
+                    nullptr, 0, nullptr, ASR::abiType::BindC,
+                    ASR::accessType::Public, ASR::deftypeType::Interface,
+                    nullptr, false, false, false, false, false, nullptr, 0,
+                    false, false, false, nullptr));
+                current_scope->add_symbol(fn_name, fnInterface);
+                pass_result.push_back(al, ASRUtils::STMT(
+                    ASR::make_SubroutineCall_t(al, loc, fnInterface, fnInterface,
+                    call_args.p, call_args.n, nullptr)));
                 remove_original_statement = true;
                 return;
             }
@@ -1385,7 +1505,7 @@ class DoConcurrentVisitor :
             std::vector<std::string> array_variables;
             // create data variable for the thread data module
             ASRUtils::ASRBuilder b(al, x.base.base.loc);
-            ASR::expr_t* data_expr = b.Variable(current_scope, current_scope->get_unique_name("data"), ASRUtils::TYPE(ASR::make_StructType_t(al, x.base.base.loc, thread_data_ext_sym)), ASR::intentType::Local);
+            ASR::expr_t* data_expr = b.Variable(current_scope, current_scope->get_unique_name("data"), ASRUtils::TYPE(ASRUtils::make_StructType_t_util(al, x.base.base.loc, thread_data_ext_sym)), ASR::intentType::Local);
             LCOMPILERS_ASSERT(data_expr != nullptr);
 
             // now create a tdata (cptr)
@@ -1602,6 +1722,26 @@ class DoConcurrentVisitor :
             current_scope = current_scope_copy;
         }
 
+        void visit_FunctionCall(const ASR::FunctionCall_t &x) {
+            SymbolTable* current_scope_copy = current_scope;
+            current_scope = ASRUtils::symbol_parent_symtab(x.m_name);
+
+            ReplaceSymbolsVisitor sym_replacer(*current_scope);
+            sym_replacer.visit_FunctionCall(x);
+
+            current_scope = current_scope_copy;
+        }
+
+        void visit_SubroutineCall(const ASR::SubroutineCall_t &x) {
+            SymbolTable* current_scope_copy = current_scope;
+            current_scope = ASRUtils::symbol_parent_symtab(x.m_name);
+
+            ReplaceStatementsVisitor stmt_replacer(*current_scope);
+            stmt_replacer.visit_SubroutineCall(x);
+
+            current_scope = current_scope_copy;
+        }
+
         void visit_DoLoop(const ASR::DoLoop_t &x) {
             ASR::DoLoop_t& xx = const_cast<ASR::DoLoop_t&>(x);
 
