@@ -1,6 +1,8 @@
 #ifndef LCOMPILERS_PASS_MANAGER_H
 #define LCOMPILERS_PASS_MANAGER_H
 
+#include <iostream>
+
 #include <libasr/asr.h>
 #include <libasr/string_utils.h>
 #include <libasr/alloc.h>
@@ -26,7 +28,6 @@
 #include <libasr/pass/replace_print_arr.h>
 #include <libasr/pass/replace_where.h>
 #include <libasr/pass/replace_print_list_tuple.h>
-#include <libasr/pass/replace_arr_slice.h>
 #include <libasr/pass/replace_flip_sign.h>
 #include <libasr/pass/replace_div_to_mul.h>
 #include <libasr/pass/replace_symbolic.h>
@@ -51,10 +52,13 @@
 #include <libasr/pass/nested_vars.h>
 #include <libasr/pass/unique_symbols.h>
 #include <libasr/pass/insert_deallocate.h>
+#include <libasr/pass/array_struct_temporary.h>
 #include <libasr/pass/replace_print_struct_type.h>
 #include <libasr/pass/promote_allocatable_to_nonallocatable.h>
 #include <libasr/pass/replace_function_call_in_declaration.h>
+#include <libasr/pass/replace_array_passed_in_function_call.h>
 #include <libasr/pass/replace_openmp.h>
+#include <libasr/pass/replace_with_compile_time_values.h>
 #include <libasr/codegen/asr_to_fortran.h>
 #include <libasr/asr_verify.h>
 #include <libasr/pickle.h>
@@ -73,10 +77,11 @@ namespace LCompilers {
         private:
 
         std::vector<std::string> _passes;
-        std::vector<std::string> _with_optimization_passes;
+        std::vector<std::string> _optimization_passes;
         std::vector<std::string> _user_defined_passes;
         std::vector<std::string> _skip_passes, _c_skip_passes;
         std::map<std::string, pass_function> _passes_db = {
+            {"replace_with_compile_time_values", &pass_replace_with_compile_time_values},
             {"do_loops", &pass_replace_do_loops},
             {"while_else", &pass_while_else},
             {"global_stmts", &pass_wrap_global_stmts},
@@ -86,7 +91,6 @@ namespace LCompilers {
             {"flip_sign", &pass_replace_flip_sign},
             {"intrinsic_function", &pass_replace_intrinsic_function},
             {"intrinsic_subroutine", &pass_replace_intrinsic_subroutine},
-            {"arr_slice", &pass_replace_arr_slice},
             {"print_arr", &pass_replace_print_arr},
             {"print_list_tuple", &pass_replace_print_list_tuple},
             {"class_constructor", &pass_replace_class_constructor},
@@ -109,23 +113,38 @@ namespace LCompilers {
             {"nested_vars", &pass_nested_vars},
             {"where", &pass_replace_where},
             {"function_call_in_declaration", &pass_replace_function_call_in_declaration},
+            {"array_passed_in_function_call", &pass_replace_array_passed_in_function_call},
             {"openmp", &pass_replace_openmp},
             {"print_struct_type", &pass_replace_print_struct_type},
             {"unique_symbols", &pass_unique_symbols},
             {"insert_deallocate", &pass_insert_deallocate},
-            {"promote_allocatable_to_nonallocatable", &pass_promote_allocatable_to_nonallocatable}
+            {"promote_allocatable_to_nonallocatable", &pass_promote_allocatable_to_nonallocatable},
+            {"array_struct_temporary", &pass_array_struct_temporary}
         };
 
         bool apply_default_passes;
         bool c_skip_pass; // This will contain the passes that are to be skipped in C
 
         public:
+        // This should be removed after a refactor to `pass_manager.h` (This action should be done using more flexible function)
+        std::vector<std::string> passes_to_skip_with_llvm;
         bool rtlib=false;
-
         void apply_passes(Allocator& al, ASR::TranslationUnit_t* asr,
                            std::vector<std::string>& passes, PassOptions &pass_options,
-                           [[maybe_unused]] diag::Diagnostics &diagnostics) {
+                           [[maybe_unused]] diag::Diagnostics &diagnostics,
+                           double &cummulative_time_taken_by_passes_in_microseconds) {
             if (pass_options.pass_cumulative) {
+                std::vector<std::string> _with_optimization_passes;
+                _with_optimization_passes.insert(
+                    _with_optimization_passes.end(),
+                    _passes.begin(),
+                    _passes.end()
+                );
+                _with_optimization_passes.insert(
+                    _with_optimization_passes.end(),
+                    _optimization_passes.begin(),
+                    _optimization_passes.end()
+                );
                 int _pass_max_idx = -1, _opt_max_idx = -1;
                 for (std::string &current_pass: passes) {
                     auto it1 = std::find(_passes.begin(), _passes.end(), current_pass);
@@ -166,6 +185,7 @@ namespace LCompilers {
                 if (pass_options.verbose) {
                     std::cerr << "ASR Pass starts: '" << passes[i] << "'\n";
                 }
+                auto t1 = std::chrono::high_resolution_clock::now();
                 _passes_db[passes[i]](al, *asr, pass_options);
 #if defined(WITH_LFORTRAN_ASSERT)
                 if (!asr_verify(*asr, true, diagnostics)) {
@@ -174,6 +194,19 @@ namespace LCompilers {
                         + passes[i]);
                 };
 #endif
+                auto t2 = std::chrono::high_resolution_clock::now();
+                if (pass_options.time_report) {
+                    int time_taken_by_current_pass = std::chrono::duration_cast<std::chrono::microseconds>(t2 - t1).count();
+                    double time_in_milliseconds = (double) time_taken_by_current_pass / 1000.0;
+                    std::string message = "";
+                    if ( time_in_milliseconds >= 1.0 ) {
+                        message = "[PASS]" + passes[i] + ": " + std::to_string(time_taken_by_current_pass / 1000) + "." + std::to_string(time_taken_by_current_pass % 1000) + " ms";
+                    } else {
+                        message = "[PASS]" + passes[i] + ": " + std::to_string(time_in_milliseconds) + " ms";
+                    }
+                    pass_options.vector_of_time_report.push_back(message);
+                    cummulative_time_taken_by_passes_in_microseconds += (double) std::chrono::duration_cast<std::chrono::microseconds>(t2 - t1).count();
+                }
                 if (pass_options.verbose) {
                     std::cerr << "ASR Pass ends: '" << passes[i] << "'\n";
                 }
@@ -208,25 +241,26 @@ namespace LCompilers {
         PassManager(): apply_default_passes{false},
             c_skip_pass{false} {
             _passes = {
-                "nested_vars",
                 "global_stmts",
-                "transform_optional_argument_functions",
                 "init_expr",
-                "forall",
+                "function_call_in_declaration",
                 "openmp",
                 "implied_do_loops",
+                "array_struct_temporary",
+                "transform_optional_argument_functions",
+                "nested_vars",
+                "forall",
                 "class_constructor",
                 "pass_list_expr",
                 "where",
-                "function_call_in_declaration",
                 "subroutine_from_function",
                 "array_op",
                 "symbolic",
                 "intrinsic_function",
                 "intrinsic_subroutine",
-                "subroutine_from_function",
                 "array_op",
                 "pass_array_by_data",
+                "array_passed_in_function_call",
                 "print_struct_type",
                 "print_arr",
                 "print_list_tuple",
@@ -235,56 +269,26 @@ namespace LCompilers {
                 "do_loops",
                 "while_else",
                 "select_case",
-                "inline_function_calls",
                 "unused_functions",
                 "unique_symbols",
                 "insert_deallocate",
             };
-
-            _with_optimization_passes = {
-                "nested_vars",
-                "global_stmts",
-                "transform_optional_argument_functions",
-                "init_expr",
-                "openmp",
-                "implied_do_loops",
-                "class_constructor",
-                "pass_list_expr",
-                "where",
-                "function_call_in_declaration",
-                "subroutine_from_function",
-                "array_op",
-                "symbolic",
-                "flip_sign",
-                "intrinsic_function",
-                "intrinsic_subroutine",
-                "subroutine_from_function",
-                "array_op",
-                "pass_array_by_data",
-                "print_struct_type",
-                "print_arr",
-                "print_list_tuple",
-                "print_struct_type",
+            _optimization_passes = {
+                "replace_with_compile_time_values",
                 "loop_vectorise",
-                "array_dim_intrinsics_update",
-                "do_loops",
-                "forall",
-                "while_else",
                 "dead_code_removal",
-                "select_case",
                 "unused_functions",
                 "sign_from_value",
                 "div_to_mul",
                 "fma",
-                "inline_function_calls",
-                "unique_symbols",
-                "insert_deallocate",
+                // "inline_function_calls",
                 "promote_allocatable_to_nonallocatable"
             };
 
             // These are re-write passes which are already handled
             // appropriately in C backend.
             _c_skip_passes = {
+                "replace_with_compile_time_values",
                 "pass_list_expr",
                 "print_list_tuple",
                 "do_loops",
@@ -297,6 +301,7 @@ namespace LCompilers {
         void parse_pass_arg(std::string& arg_pass, std::string& skip_pass) {
             _user_defined_passes.clear();
             _skip_passes.clear();
+            _skip_passes.insert(_skip_passes.end(),passes_to_skip_with_llvm.begin(), passes_to_skip_with_llvm.end());
             _parse_pass_arg(arg_pass, _user_defined_passes);
             _parse_pass_arg(skip_pass, _skip_passes);
         }
@@ -304,17 +309,30 @@ namespace LCompilers {
         void apply_passes(Allocator& al, ASR::TranslationUnit_t* asr,
                           PassOptions& pass_options,
                           diag::Diagnostics &diagnostics) {
+            double cummulative_time_taken_by_passes_in_microseconds = 0.0;
+            auto t1 = std::chrono::high_resolution_clock::now();
             if( !_user_defined_passes.empty() ) {
                 apply_passes(al, asr, _user_defined_passes, pass_options,
-                    diagnostics);
+                    diagnostics, cummulative_time_taken_by_passes_in_microseconds);
             } else if( apply_default_passes ) {
-                if( pass_options.fast ) {
-                    apply_passes(al, asr, _with_optimization_passes, pass_options,
-                        diagnostics);
-                } else {
-                    apply_passes(al, asr, _passes, pass_options, diagnostics);
+                apply_passes(al, asr, _passes, pass_options, diagnostics, cummulative_time_taken_by_passes_in_microseconds);
+                if (pass_options.fast ){
+                    apply_passes(al, asr, _optimization_passes, pass_options, diagnostics, cummulative_time_taken_by_passes_in_microseconds);
                 }
             }
+            auto t2 = std::chrono::high_resolution_clock::now();
+            if (pass_options.time_report) {
+                int overall_time_in_passes = std::chrono::duration_cast<std::chrono::microseconds>(t2 - t1).count();
+                overall_time_in_passes = overall_time_in_passes - cummulative_time_taken_by_passes_in_microseconds;
+                double time_in_milliseconds = (double) overall_time_in_passes / 1000.0;
+                std::string message = "";
+                if ( time_in_milliseconds >= 1.0 ) {
+                    message = "[PASS]other processing time: " + std::to_string(overall_time_in_passes / 1000) + "." + std::to_string(overall_time_in_passes % 1000) + " ms";
+                } else {
+                    message = "[PASS]other processing time: " + std::to_string(time_in_milliseconds) + " ms";
+                }
+                pass_options.vector_of_time_report.push_back(message);
+            }
         }
 
         void dump_all_passes(Allocator& al, ASR::TranslationUnit_t* asr,
@@ -322,23 +340,31 @@ namespace LCompilers {
                            [[maybe_unused]] diag::Diagnostics &diagnostics, LocationManager &lm) {
             std::vector<std::string> passes;
             if (pass_options.fast) {
-                passes = _with_optimization_passes;
+                passes = _passes;
+                passes.insert(
+                    passes.end(),
+                    _optimization_passes.begin(),
+                    _optimization_passes.end()
+                );
             } else {
                 passes = _passes;
             }
+            size_t pass_cnt_asr_dump = 0, pass_cnt_fortran_dump = 0;
             for (size_t i = 0; i < passes.size(); i++) {
                 // TODO: rework the whole pass manager: construct the passes
                 // ahead of time (not at the last minute), and remove this much
                 // earlier
                 // Note: this is not enough for rtlib, we also need to include
                 // it
+                if( std::find(_skip_passes.begin(), _skip_passes.end(), passes[i]) != _skip_passes.end()) continue;
                 if (pass_options.verbose) {
                     std::cerr << "ASR Pass starts: '" << passes[i] << "'\n";
                 }
                 _passes_db[passes[i]](al, *asr, pass_options);
                 if (pass_options.dump_all_passes) {
-                    std::string str_i = std::to_string(i+1);
-                    if ( i < 9 )  str_i = "0" + str_i;
+                    std::string str_i = std::to_string(pass_cnt_asr_dump+1);
+                    if ( pass_cnt_asr_dump < 9 )  str_i = "0" + str_i;
+                    ++pass_cnt_asr_dump;
                     if (pass_options.json) {
                         std::ofstream outfile ("pass_json_" + str_i + "_" + passes[i] + ".json");
                         outfile << pickle_json(*asr, lm, pass_options.no_loc, pass_options.with_intrinsic_mods) << "\n";
@@ -367,8 +393,9 @@ namespace LCompilers {
                         throw LCompilersException("Fortran code could not be generated after pass: "
                         + passes[i]);
                     }
-                    std::string str_i = std::to_string(i+1);
-                    if ( i < 9 )  str_i = "0" + str_i;
+                    std::string str_i = std::to_string(pass_cnt_fortran_dump+1);
+                    if ( pass_cnt_fortran_dump < 9 )  str_i = "0" + str_i;
+                    ++pass_cnt_fortran_dump;
                     std::ofstream outfile ("pass_fortran_" + str_i + "_" + passes[i] + ".f90");
                     outfile << "! Fortran code after applying the pass: " << passes[i]
                         << "\n" << fortran_code.result << "\n";
