@@ -66,11 +66,29 @@ class IsAllocatedCalled: public ASR::CallReplacerOnExpressionsVisitor<IsAllocate
             }
         }
 
+        bool is_array_size_called_on_pointer(ASR::dimension_t* m_dims, size_t n_dims) {
+            for( size_t i = 0; i < n_dims; i++ ) {
+                #define check_pointer_in_array_size(expr) if( expr && ASR::is_a<ASR::ArraySize_t>(*expr) ) { \
+                    ASR::ArraySize_t* array_size_t = ASR::down_cast<ASR::ArraySize_t>(expr); \
+                    if( ASRUtils::is_pointer(ASRUtils::expr_type(array_size_t->m_v)) ) { \
+                        return true; \
+                    } \
+                } \
+
+                check_pointer_in_array_size(m_dims[i].m_start)
+                check_pointer_in_array_size(m_dims[i].m_length)
+
+            }
+
+            return false;
+        }
+
         void visit_Allocate(const ASR::Allocate_t& x) {
             for( size_t i = 0; i < x.n_args; i++ ) {
                 ASR::alloc_arg_t alloc_arg = x.m_args[i];
                 if( !ASRUtils::is_dimension_dependent_only_on_arguments(
-                        alloc_arg.m_dims, alloc_arg.n_dims) ) {
+                        alloc_arg.m_dims, alloc_arg.n_dims, true) ||
+                    is_array_size_called_on_pointer(alloc_arg.m_dims, alloc_arg.n_dims) ) {
                     if( ASR::is_a<ASR::Var_t>(*alloc_arg.m_a) ) {
                         scope2var[current_scope].push_back(
                                 ASR::down_cast<ASR::Var_t>(alloc_arg.m_a)->m_v);
@@ -108,6 +126,8 @@ class PromoteAllocatableToNonAllocatable:
                     ASRUtils::is_array(ASRUtils::expr_type(alloc_arg.m_a)) &&
                     ASR::is_a<ASR::Variable_t>(
                         *ASR::down_cast<ASR::Var_t>(alloc_arg.m_a)->m_v) &&
+                    !ASR::is_a<ASR::Module_t>(
+                        *ASRUtils::get_asr_owner(ASR::down_cast<ASR::Var_t>(alloc_arg.m_a)->m_v)) &&
                     ASRUtils::expr_intent(alloc_arg.m_a) == ASRUtils::intent_local &&
                     ASRUtils::is_dimension_dependent_only_on_arguments(
                         alloc_arg.m_dims, alloc_arg.n_dims) &&
@@ -241,8 +261,10 @@ class FixArrayPhysicalCastVisitor: public ASR::CallReplacerOnExpressionsVisitor<
 
         Allocator& al;
         FixArrayPhysicalCast replacer;
+        bool remove_original_stmt;
 
-        FixArrayPhysicalCastVisitor(Allocator& al_): al(al_), replacer(al_) {}
+        FixArrayPhysicalCastVisitor(Allocator& al_):
+            al(al_), replacer(al_), remove_original_stmt(false) {}
 
         void call_replacer() {
             replacer.current_expr = current_expr;
@@ -259,6 +281,39 @@ class FixArrayPhysicalCastVisitor: public ASR::CallReplacerOnExpressionsVisitor<
             xx.m_args = subrout_call->m_args;
             xx.n_args = subrout_call->n_args;
         }
+
+        void visit_Associate(const ASR::Associate_t& x) {
+            if( ASRUtils::is_fixed_size_array(
+                    ASRUtils::expr_type(x.m_value)) ) {
+                ASR::Associate_t& xx = const_cast<ASR::Associate_t&>(x);
+                xx.m_value = ASRUtils::EXPR(ASRUtils::make_ArrayPhysicalCast_t_util(
+                    al, x.m_value->base.loc, xx.m_value,
+                    ASRUtils::extract_physical_type(ASRUtils::expr_type(xx.m_value)),
+                    ASR::array_physical_typeType::DescriptorArray,
+                    ASRUtils::duplicate_type(al, ASRUtils::expr_type(x.m_value),
+                    nullptr, ASR::array_physical_typeType::DescriptorArray, true), nullptr));
+            } else if( ASRUtils::is_fixed_size_array(
+                        ASRUtils::expr_type(x.m_target)) ) {
+                remove_original_stmt = true;
+            }
+        }
+
+        void transform_stmts(ASR::stmt_t **&m_body, size_t &n_body) {
+            bool remove_original_stmt_copy = remove_original_stmt;
+            Vec<ASR::stmt_t*> body;
+            body.reserve(al, n_body);
+            for (size_t i = 0; i < n_body; i++) {
+                remove_original_stmt = false;
+                visit_stmt(*m_body[i]);
+                if( !remove_original_stmt ) {
+                    body.push_back(al, m_body[i]);
+                    remove_original_stmt = false;
+                }
+            }
+            m_body = body.p;
+            n_body = body.size();
+            remove_original_stmt = remove_original_stmt_copy;
+        }
 };
 
 void pass_promote_allocatable_to_nonallocatable(
