@@ -9,6 +9,7 @@
 
 #include <vector>
 #include <string>
+#include <set>
 
 /*
 Need for the pass
@@ -78,9 +79,11 @@ class ReplacePresentCalls: public ASR::BaseExprReplacer<ReplacePresentCalls> {
             ASR::symbol_t* present_arg = ASR::down_cast<ASR::Var_t>(x->m_args[0])->m_v;
             size_t i;
             for( i = 0; i < f->n_args; i++ ) {
-                if( ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v == present_arg ) {
-                    i++;
-                    break;
+                if( ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v ) {
+                    if( ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v == present_arg ) {
+                        i++;
+                        break;
+                    }
                 }
             }
 
@@ -338,6 +341,8 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
         ASR::Variable_t* v = ASR::down_cast<ASR::Variable_t>(x.m_name);
         LCOMPILERS_ASSERT(ASR::is_a<ASR::FunctionType_t>(*v->m_type));
         func_sym = ASRUtils::symbol_get_past_external(v->m_type_declaration);
+        v->m_type = ASRUtils::duplicate_type(al, ASR::down_cast<ASR::Function_t>(
+            ASRUtils::symbol_get_past_external(v->m_type_declaration))->m_function_signature);
     }
     bool is_nopass { false };
     bool is_class_procedure { false };
@@ -372,23 +377,23 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
     // which is depicted in `func.n_args` while isn't depicted in
     // `x.n_args` (as it only represents the "FunctionCall" arguments)
     // hence to adjust for that, `is_method` introduces an offset
-    bool is_method = is_class_procedure && (!is_nopass);
+    int is_method = is_class_procedure && (!is_nopass);
 
     new_args.reserve(al, func->n_args);
-    for( size_t i = 0, j = 0; j < func->n_args; j++, i++ ) {
-        LCOMPILERS_ASSERT(i < x.n_args + is_method);
+    for( int i = 0, j = 0; j < (int)func->n_args; j++, i++ ) {
         if( std::find(sym2optionalargidx[func_sym].begin(),
                       sym2optionalargidx[func_sym].end(), j)
             != sym2optionalargidx[func_sym].end() ) {
             ASR::Variable_t* func_arg_j = ASRUtils::EXPR2VAR(func->m_args[j]);
-            if( x.m_args[i - is_method].m_value == nullptr ) {
+            if( i - is_method >= (int)x.n_args || x.m_args[i - is_method].m_value == nullptr ) {
                 std::string m_arg_i_name = scope->get_unique_name("__libasr_created_variable_");
                 ASR::ttype_t* arg_type = func_arg_j->m_type;
+                ASR::symbol_t* arg_decl = func_arg_j->m_type_declaration;
                 if( ASR::is_a<ASR::Array_t>(*arg_type) ) {
                     ASR::Array_t* array_t = ASR::down_cast<ASR::Array_t>(arg_type);
                     Vec<ASR::dimension_t> dims;
                     dims.reserve(al, array_t->n_dims);
-                    for( size_t i = 0; i < array_t->n_dims; i++ ) {
+                    for( int i = 0; i < (int)array_t->n_dims; i++ ) {
                         ASR::dimension_t dim;
                         dim.m_length = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, arg_type->base.loc, 1,
                                             ASRUtils::TYPE(ASR::make_Integer_t(al, arg_type->base.loc, 4))));
@@ -401,7 +406,7 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
                                 array_t->m_type, dims.p, dims.size(), ASR::array_physical_typeType::FixedSizeArray));
                 }
                 ASR::expr_t* m_arg_i = PassUtils::create_auxiliary_variable(
-                    x.m_args[i - is_method].loc, m_arg_i_name, al, scope, arg_type);
+                    x.m_args[i - is_method].loc, m_arg_i_name, al, scope, arg_type, ASR::intentType::Local, arg_decl);
                 arg_type = ASRUtils::expr_type(m_arg_i);
                 if( ASRUtils::is_array(arg_type) &&
                     ASRUtils::extract_physical_type(arg_type) !=
@@ -422,19 +427,19 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
             ASR::ttype_t* logical_t = ASRUtils::TYPE(ASR::make_Logical_t(al,
                                         x.m_args[i - is_method].loc, 4));
             ASR::expr_t* is_present = nullptr;
-            if( x.m_args[i - is_method].m_value == nullptr ) {
+            if( i - is_method >= (int)x.n_args || x.m_args[i - is_method].m_value == nullptr ) {
                 is_present = ASRUtils::EXPR(ASR::make_LogicalConstant_t(
-                    al, x.m_args[i - is_method].loc, false, logical_t));
+                    al, x.m_args[0].loc, false, logical_t));
             } else {
                 if( owning_function != nullptr ) {
                     size_t k;
                     bool k_found = false;
+                    ASR::expr_t* original_expr = nullptr;
+                    if (ASR::is_a<ASR::ArrayPhysicalCast_t>(*x.m_args[i - is_method].m_value)) {
+                        ASR::ArrayPhysicalCast_t *x_array_cast = ASR::down_cast<ASR::ArrayPhysicalCast_t>(x.m_args[i - is_method].m_value);
+                        original_expr = x_array_cast->m_arg;
+                    }
                     for( k = 0; k < owning_function->n_args; k++ ) {
-                        ASR::expr_t* original_expr = nullptr;
-                        if (ASR::is_a<ASR::ArrayPhysicalCast_t>(*x.m_args[i - is_method].m_value)) {
-                            ASR::ArrayPhysicalCast_t *x_array_cast = ASR::down_cast<ASR::ArrayPhysicalCast_t>(x.m_args[i - is_method].m_value);
-                            original_expr = x_array_cast->m_arg;
-                        }
                         if( original_expr && ASR::is_a<ASR::Var_t>(*original_expr) && ASR::down_cast<ASR::Var_t>(owning_function->m_args[k])->m_v ==
                             ASR::down_cast<ASR::Var_t>(original_expr)->m_v ) {
                             k_found = true;
@@ -456,13 +461,24 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
                 }
 
                 if( is_present == nullptr ) {
-                    is_present = ASRUtils::EXPR(ASR::make_LogicalConstant_t(
-                        al, x.m_args[i - is_method].loc, true, logical_t));
+                    ASR::expr_t* arg_i = x.m_args[i - is_method].m_value;
+                    const Location& loc = arg_i->base.loc;
+                    LCOMPILERS_ASSERT(arg_i != nullptr);
+                    if( ASRUtils::is_pointer(ASRUtils::expr_type(arg_i)) ) {
+                        ASR::ttype_t* associated_type_ = ASRUtils::TYPE(
+                            ASR::make_Logical_t(al, loc, 4));
+                        is_present = ASRUtils::EXPR(ASR::make_PointerAssociated_t(
+                            al, loc, arg_i, nullptr, associated_type_, nullptr));
+                    } else {
+                        is_present = ASRUtils::EXPR(ASR::make_LogicalConstant_t(
+                            al, loc, true, logical_t));
+                    }
                 }
             }
             ASR::call_arg_t present_arg;
             present_arg.loc = x.m_args[i - is_method].loc;
-            if( x.m_args[i - is_method].m_value &&
+            if( i - is_method < (int)x.n_args &&
+                x.m_args[i - is_method].m_value &&
                 ASRUtils::is_allocatable(x.m_args[i - is_method].m_value) &&
                 !ASRUtils::is_allocatable(func_arg_j->m_type) ) {
                 ASR::expr_t* is_allocated = ASRUtils::EXPR(ASR::make_IntrinsicImpureFunction_t(
@@ -474,7 +490,8 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
             present_arg.m_value = is_present;
             new_args.push_back(al, present_arg);
             j++;
-        } else if (!is_method) {
+        } else {
+            if(i - is_method < 0) continue;
             // not needed to have `i - is_method` can be simply
             // `i` as well, just for consistency with code above
             new_args.push_back(al, x.m_args[i - is_method]);
